{********************************************
; File: GSOS.p
;
;
; Copyright Apple Computer, Inc. 1986-91
; All Rights Reserved
;
********************************************}

unit gsos;

  interface

    uses types;

    const

{
    Read/Write enable bit Codes
    for CreateRec/OpenRec access and requestAccess fields
}

      readenableallowwrite = $0000;
      readenable = $0001;
      writeenable = $0002;
      readwriteenable = $0003;
      fileinvisible = $0004; { Invisible bit }
      backupneeded = $0020; { backup needed bit: CreateRec/ OpenRec access
                             field. (Must be 0 in requestAccess field ) }
      renameenable = $0040; { rename enable bit: CreateRec/ OpenRec access and
                             requestAccess fields }
      destroyenable = $0080; { destroy enable bit: CreateRec/ OpenRec access and
                              requestAccess fields }
      startplus = $0000; { base -> setMark = displacement }
      eofminus = $0001; { base -> setMark = eof - displacement }
      markplus = $0002; { base -> setMark = mark + displacement }
      markminus = $0003; { base -> setMark = mark - displacement }

      { cachePriority Codes }
      cacheoff = $0000; { do not cache blocks invloved in this read }
      cacheon = $0001; { cache blocks invloved in this read if possible }

      { Error Codes }
      badsystemcall = $0001; { bad system call number }
      invalidpcount = $0004; { invalid parameter count }
      gsosactive = $0007; { GS/OS already active }
      {$IFC UNDEFINED devNotFound } { device not found }
      devnotfound = $0010;
      {$SETC devNotFound := 0}
      {$ENDC}
      invaliddevnum = $0011; { invalid device number }
      drvrbadreq = $20; { bad request or command }
      drvrbadcode = $0021; { bad control or status code }
      drvrbadparm = $0022; { bad call parameter }
      drvrnotopen = $0023; { character device not open }
      drvrprioropen = $0024; { character device already open }
      irqtablefull = $0025; { interrupt table full }
      drvrnoresrc = $0026; { resources not available }
      drvrioerror = $0027; { I/O error }
      drvrnodevice = $0028; { device not connected }
      drvrbusy = $0029; { call aborted; driver is busy }
      drvrwrtprot = $002B; { device is write protected }
      drvrbadcount = $002C; { invalid byte count }
      drvrbadblock = $002D; { invalid block address }
      drvrdiskswitch = $002E; { disk has been switched }
      drvroffline = $002F; { device off line/ no media present }
      badpathsyntax = $0040; { invalid pathname syntax }
	  toomanyfilesopen = $0042; { too many files open on server volume }
      invalidrefnum = $0043; { invalid reference number }
      {$IFC UNDEFINED pathNotFound } { subdirectory does not exist }
      pathnotfound = $44;
      {$SETC pathNotFound := 0}
      {$ENDC}
      volnotfound = $0045; { volume not found }
      {$IFC UNDEFINED fileNotFound } { file not found }
      filenotfound = $0046;
      {$SETC fileNotFound := 0}
      {$ENDC}
      duppathname = $0047; { create or rename with existing name }
      volumefull = $0048; { volume full error }
      voldirfull = $0049; { volume directory full }
      badfileformat = $004A; { version error (incompatible file format) }
      {$IFC UNDEFINED badStoreType } { unsupported (or incorrect) storage type }
      badstoretype = $004B;
      {$SETC badStoreType := 0}
      {$ENDC}
      {$IFC UNDEFINED eofEncountered } { end-of-file encountered }
      eofencountered = $004C;
      {$SETC eofEncountered := 0}
      {$ENDC}
      outofrange = $004D; { position out of range }
      invalidaccess = $004E; { access not allowed }
      bufftoosmall = $004F; { buffer too small }
      filebusy = $0050; { file is already open }
      direrror = $0051; { directory error }
      unknownvol = $0052; { unknown volume type }
      {$IFC UNDEFINED paramRangeErr } { parameter out of range }
      paramrangeerr = $0053;
      {$SETC paramRangeErr := 0}
      {$ENDC}
      outofmem = $0054; { out of memory }
      dupvolume = $0057; { duplicate volume name }
      notblockdev = $0058; { not a block device }
      {$IFC UNDEFINED invalidLevel } { specifield level outside legal range }
      invalidlevel = $0059;
      {$SETC invalidLevel := 0}
      {$ENDC}
      damagedbitmap = $005A; { block number too large }
      badpathnames = $005B; { invalid pathnames for ChangePath }
      notsystemfile = $005C; { not an executable file }
      osunsupported = $005D; { Operating System not supported }
      {$IFC UNDEFINED stackOverflow } { too many applications on stack }
      stackoverflow = $005F;
      {$SETC stackOverflow := 0}
      {$ENDC}
      dataunavail = $0060; { Data unavailable }
      endofdir = $0061; { end of directory has been reached }
      invalidclass = $0062; { invalid FST call class }
      resforknotfound = $0063; { file does not contain required resource }
      invalidfstid = $0064; { error - FST ID is invalid }
	  invalidFSTop = $0065; { invalid FST operation }
	  fstCaution = $0066; { FST handled call, but result is wierd }
      devnameerr = $0067; { device exists with same name as replacement name }
	  devListFull = $0068; { device list is full }
	  supListFull = $0069; { supervisor list is full }
	  fstError = $006a; { generic FST error
      resexistserr = $0070; { cannot expand file, resource already exists }
      resadderr = $0071; { cannot add resource fork to this type file }
	  networkError = $0088; { generic network error }

      { fileSys IDs }
      prodosfsid = $0001; { ProDOS/SOS }
      dos33fsid = $0002; { DOS 3.3 }
      dos32fsid = $0003; { DOS 3.2 }
      dos31fsid = $0003; { DOS 3.1 }
      appleiipascalfsid = $0004; { Apple II Pascal }
      mfsfsid = $0005; { Macintosh (flat file system) }
      hfsfsid = $0006; { Macintosh (hierarchical file system) }
      lisafsid = $0007; { Lisa file system }
      applecpmfsid = $0008; { Apple CP/M }
      charfstfsid = $0009; { Character FST }
      msdosfsid = $000A; { MS/DOS }
      highsierrafsid = $000B; { High Sierra }
      iso9660fsid = $000C; { ISO 9660 }
      applesharefsid = $000D; { ISO 9660 }

      { FSTInfo.attributes Codes }
      characterfst = $4000; { character FST }
      ucfst = $8000; { SCM should upper case pathnames before passing them to
                      the FST }

      { QuitRec.flags Codes }
      onstack = $8000; { place state information about quitting program on the
                        quit return stack }
      restartable = $4000; { the quitting program is capable of being restarted
                            from its dormant memory }

      { storageType Codes }
      seedling = $0001; { standard file with seedling structure }
      standardfile = $0001; { standard file type (no resource fork) }
      sapling = $0002; { standard file with sapling structure }
      tree = $0003; { standard file with tree structure }
      pascalregion = $0004; { UCSD Pascal region on a partitioned disk }
      extendedfile = $0005; { extended file type (with resource fork) }
      directoryfile = $000D; { volume directory or subdirectory file }

      { version Codes }
      minorrelnummask = $00FF; { minor release number }
      majorrelnummask = $7F00; { major release number }
      finalrelnummask = $8000; { final release number }

      { Other Constants }
      isfileextended = $8000; { GetDirEntryGS }

      { DControl Codes }
      resetdevice = $0000;
      formatdevice = $0001;
      eject = $0002;
      setconfigparameters = $0003;
      setwaitstatus = $0004;
      setformatoptions = $0005;
      assignpartitionowner = $0006;
      armsignal = $0007;
      disarmsignal = $0008;
      setpartitionmap = $0009;

    type
      gsstring255hndl = ^gsstring255ptr;
      gsstring255ptr = ^gsstring255;
      gsstring255 = record
                      length: integer; { Number of Chars in text field }
                      text: packed array [1..255] of char;
                    end;
      gsstring255hndlptr = ^gsstring255hndl;

      gsstring32hndl = ^gsstring32ptr;
      gsstring32ptr = ^gsstring32;
      gsstring32 = record
                     length: integer; { Number of characters in text field }
                     text: packed array [1..32] of char;
                   end;
      resultbuf255hndl = ^resultbuf255ptr;
      resultbuf255ptr = ^resultbuf255;
      resultbuf255 = record
                       bufsize: integer;
                       bufstring: gsstring255;
                     end;
      resultbuf255hndlptr = ^resultbuf255hndl;

      resultbuf32hndl = ^resultbuf32ptr;
      resultbuf32ptr = ^resultbuf32;
      resultbuf32 = record
                      bufsize: integer;
                      bufstring: gsstring32;
                    end;
      changepathrecptrgs = ^changepathrecgs;
      changepathrecgs = record
                          pcount: integer;
                          pathname: gsstring255ptr;
                          newpathname: gsstring255ptr;
						  flags : Integer;
                        end;
      createrecptrgs = ^createrecgs;
      createrecgs = record
                      pcount: integer;
                      pathname: gsstring255ptr;
                      access: integer;
                      filetype: integer;
                      auxtype: longint;
                      storagetype: integer;
                      eof: longint;
                      resourceeof: longint;
                    end;
      daccessrecptrgs = ^daccessrecgs;
      daccessrecgs = record
                       pcount: integer;
                       devnum: integer;
                       code: integer;
                       list: ptr;
                       requestcount: longint;
                       transfercount: longint;
                     end;
      devnumrecptrgs = ^devnumrecgs;
      devnumrecgs = record
                      pcount: integer;
                      devname: gsstring32ptr;
                      devnum: integer;
                    end;
      dinforecptrgs = ^dinforecgs;
      dinforecgs = record
                     pcount: integer; { minimum = 2 }
                     devnum: integer;
                     devname: resultbuf32ptr;
                     characteristics: integer;
                     totalblocks: longint;
                     slotnum: integer;
                     unitnum: integer;
                     version: integer;
                     deviceid: integer;
                     headlink: integer;
                     forwardlink: integer;
                     extendeddibptr: ptr;
                   end;
      diorecptrgs = ^diorecgs;
      diorecgs = record
                   pcount: integer;
                   devnum: integer;
                   buffer: ptr;
                   requestcount: longint;
                   startingblock: longint;
                   blocksize: integer;
                   transfercount: longint;
                 end;
      direntryrecptrgs = ^direntryrecgs;
      direntryrecgs = record
                        pcount: integer;
                        refnum: integer;
                        flags: integer;
                        base: integer;
                        displacement: integer;
                        name: resultbuf255ptr;
                        entrynum: integer;
                        filetype: integer;
                        eof: longint;
                        blockcount: longint;
                        createdatetime: timerec;
                        moddatetime: timerec;
                        access: integer;
                        auxtype: longint;
                        filesysid: integer;
                        optionlist: resultbuf255ptr;
                        resourceeof: longint;
                        resourceblocks: longint;
                      end;
      drenamerecgsptr = ^drenamerecgs;
      drenamerecgs = record
                       pcount: integer;
                       devnum: integer;
                       strptr: gsstring32ptr;
                     end;
      expandpathrecptrgs = ^expandpathrecgs;
      expandpathrecgs = record
                          pcount: integer;
                          inputpath: gsstring255ptr;
                          outputpath: resultbuf255ptr;
                          flags: integer;
                        end;
      fileinforecptrgs = ^fileinforecgs;
      fileinforecgs = record
                        pcount: integer;
                        pathname: gsstring255ptr;
                        access: integer;
                        filetype: integer;
                        auxtype: longint;
                        storagetype: integer; { must be 0 for SetFileInfo }
                        createdatetime: timerec;
                        moddatetime: timerec;
                        optionlist: resultbuf255ptr;
                        eof: longint;
                        blocksused: longint; { must be 0 for SetFileInfo }
                        resourceeof: longint; { must be 0 for SetFileInfo }
                        resourceblocks: longint; { must be 0 for SetFileInfo }
                      end;
      formatrecptrgs = ^formatrecgs;
      formatrecgs = record
                      pcount: integer;
                      devname: gsstring32ptr; { device name pointer }
                      volname: gsstring32ptr; { volume name pointer }
                      filesysid: integer; { file system ID }
                      reqfilesysid: integer; { in; }
					  flags: integer;
					  realvolname: resultbuf255ptr;
                    end;
      fstinforecptrgs = ^fstinforecgs;
      fstinforecgs = record
                       pcount: integer;
                       fstnum: integer;
                       filesysid: integer;
                       fstname: resultbuf255ptr;
                       version: integer;
                       attributes: integer;
                       blocksize: integer;
                       maxvolsize: longint;
                       maxfilesize: longint;
                     end;
      interruptrecptrgs = ^interruptrecgs;
      interruptrecgs = record
                         pcount: integer;
                         intnum: integer;
                         vrn: integer; { used only by BindInt }
                         intcode: procptr; { used only by BindInt }
                       end;
      iorecptrgs = ^iorecgs;
      iorecgs = record
                  pcount: integer;
                  refnum: integer;
                  databuffer: ptr;
                  requestcount: longint;
                  transfercount: longint;
                  cachepriority: integer;
                end;
	  judgenamerecptrgs = ^judgenamerecgs;
	  judgenamerecgs = record
	  					pcount: integer;
						filesysid: integer;
						nametype: integer;
						syntax:	stringptr;
						maxlen: integer;
						name: resultbuf255ptr;
						nameflags: integer;
					end;
      levelrecptrgs = ^levelrecgs;
      levelrecgs = record
                     pcount: integer;
                     level: integer;
					 levelmode: integer;
                   end;
      namerecptrgs = ^namerecgs;
      namerecgs = record
                    pcount: integer;
                    pathname: gsstring255ptr; { full pathname or a filename
                                               depending on call }
                  end;
      notifyprocrecgsptr = ^notifyprocrecgs;
      notifyprocrecgs = record
                          pcount: integer;
                          procpointer: procptr;
                        end;
      getnamerecptrgs = ^getnamerecgs;
      getnamerecgs = record
                       pcount: integer;
                       databuffer: resultbuf255ptr; { full pathname or a
                                                     filename depending on call }
					   userid: integer;
                     end;
      newlinerecptrgs = ^newlinerecgs;
      newlinerecgs = record
                       pcount: integer;
                       refnum: integer;
                       enablemask: integer;
                       numchars: integer;
                       newlinetable: ptr;
                     end;
      openrecptrgs = ^openrecgs;
      openrecgs = record
                    pcount: integer;
                    refnum: integer;
                    pathname: gsstring255ptr;
                    requestaccess: integer;
                    resourcenumber: integer; { For extended files:
                                              dataFork/resourceFork }
                    access: integer; { Value of file's access attribute }
                    filetype: integer; { Value of file's fileType attribute }
                    auxtype: longint;
                    storagetype: integer;
                    createdatetime: timerec;
                    moddatetime: timerec;
                    optionlist: resultbuf255ptr;
                    eof: longint;
                    blocksused: longint;
                    resourceeof: longint;
                    resourceblocks: longint;
                  end;
      osshutdownrecptrgs = ^osshutdownrecgs;
      osshutdownrecgs = record
                          pcount: integer;
                          shutdownflag: integer;
                        end;
      positionrecptrgs = ^positionrecgs;
      positionrecgs = record
                        pcount: integer;
                        refnum: integer;
                        position: longint;
                      end;
      eofrecptrgs = ^eofrecgs;
      eofrecgs = record
                   pcount: integer;
                   refnum: integer;
                   eof: longint;
                 end;
      prefixrecptrgs = ^prefixrecgs;
      prefixrecgs = record
                      pcount: integer;
                      prefixnum: integer;
                      case integer of
                        0:
                          (getprefix: resultbuf255ptr; );
                        1:
                          (setprefix: gsstring255ptr; );
                    end;
      quitrecptrgs = ^quitrecgs;
      quitrecgs = record
                    pcount: integer;
                    pathname: gsstring255ptr; { pathname of next app to run }
                    flags: integer;
                  end;
      refnumrecptrgs = ^refnumrecgs;
      refnumrecgs = record
                      pcount: integer;
                      refnum: integer;
                    end;
      getrefnumrecptrgs = ^getrefnumrecgs;
      getrefnumrecgs = record
                         pcount: integer;
                         pathname: gsstring255ptr;
                         refnum: integer;
                         access: integer;
                         resnum: integer;
                         casesense: boolean;
                         displacement: integer;
                       end;
      stdrefnumrecgsptr = ^stdrefnumrecgs;
      stdrefnumrecgs = record
                         pcount: integer;
                         prefixnum: integer;
                         refnum: integer;
                       end;
      sessionstatusrecptrgs = ^sessionstatusrecgs;
      sessionstatusrecgs = record
                             pcount: integer; { in: min = 1 }
                             status: integer; { out: }
                           end;
      setpositionrecptrgs = ^setpositionrecgs;
      setpositionrecgs = record
                           pcount: integer;
                           refnum: integer;
                           base: integer;
                           displacement: longint;
                         end;
      sysprefsrecptrgs = ^sysprefsrecgs;
      sysprefsrecgs = record
                        pcount: integer;
                        preferences: integer;
                      end;
      versionrecptrgs = ^versionrecgs;
      versionrecgs = record
                       pcount: integer;
                       version: integer;
                     end;
      volumerecptrgs = ^volumerecgs;
      volumerecgs = record
                      pcount: integer;
                      devname: gsstring32ptr;
                      volname: resultbuf255ptr;
                      totalblocks: longint;
                      freeblocks: longint;
                      filesysid: integer;
                      blocksize: integer;
					  characteristics: integer;
					  deviceid: integer;
                    end;
      refinforecgsptr = ^refinforecgs;
      refinforecgs = record
                       pcount: integer;
                       refnum: integer;
                       access: integer;
                       pathname: resultbuf255ptr;
					   resourcenumber: integer;
					   level: integer;
                     end;
      filerecgs = record
                    case integer of
                      0:
                        (derec: direntryrecgs);
                      1:
                        (iorec: iorecgs);
                      2:
                        (nlrec: newlinerecgs);
                      3:
                        (oprec: openrecgs);
                      4:
                        (porec: positionrecgs);
                      5:
                        (eorec: eofrecgs);
                      6:
                        (rnrec: refnumrecgs);
                      7:
                        (sprec: setpositionrecgs);
                  end;

    procedure addnotifyprocgs(var pblockptr: notifyprocrecgsptr);
      inline $2034F4, $E100B022, $8F, '_toolErr';

    procedure beginsessiongs(var pblockptr: sessionstatusrecgs);
      inline $201DF4, $E100B022, $8F, '_toolErr';

    procedure bindintgs(var pblockptr: interruptrecgs);
      inline $2031F4, $E100B022, $8F, '_toolErr';

    procedure changepathgs(var pblockptr: changepathrecgs);
      inline $2004F4, $E100B022, $8F, '_toolErr';

    procedure clearbackupbitgs(var pblockptr: namerecgs);
      inline $200BF4, $E100B022, $8F, '_toolErr';

    procedure closegs(var pblockptr: refnumrecgs);
      inline $2014F4, $E100B022, $8F, '_toolErr';

    procedure creategs(var pblockptr: createrecgs);
      inline $2001F4, $E100B022, $8F, '_toolErr';

    procedure dcontrolgs(var pblockptr: daccessrecgs);
      inline $202EF4, $E100B022, $8F, '_toolErr';

    procedure delnotifyprocgs(var pblockptr: notifyprocrecgsptr);
      inline $2035F4, $E100B022, $8F, '_toolErr';

    procedure destroygs(var pblockptr: namerecgs);
      inline $2002F4, $E100B022, $8F, '_toolErr';

    procedure dinfogs(var pblockptr: dinforecgs);
      inline $202CF4, $E100B022, $8F, '_toolErr';

    procedure dreadgs(var pblockptr: diorecgs);
      inline $202FF4, $E100B022, $8F, '_toolErr';

    procedure drenamegs(var pblockptr: drenamerecgs);
      inline $2036F4, $E100B022, $8F, '_toolErr';

    procedure dstatusgs(var pblockptr: daccessrecgs);
      inline $202DF4, $E100B022, $8F, '_toolErr';

    procedure dwritegs(var pblockptr: diorecgs);
      inline $2030F4, $E100B022, $8F, '_toolErr';

    procedure endsessiongs(var pblockptr: sessionstatusrecgs);
      inline $201EF4, $E100B022, $8F, '_toolErr';

    procedure erasediskgs(var pblockptr: formatrecgs);
      inline $2025F4, $E100B022, $8F, '_toolErr';

    procedure expandpathgs(var pblockptr: expandpathrecgs);
      inline $200EF4, $E100B022, $8F, '_toolErr';

    procedure flushgs(var pblockptr: refnumrecgs);
      inline $2015F4, $E100B022, $8F, '_toolErr';

    procedure formatgs(var pblockptr: formatrecgs);
      inline $2024F4, $E100B022, $8F, '_toolErr';

    procedure fstspecific(pblockptr: ptr);
      inline $2033F4, $E100B022, $8F, '_toolErr';

    procedure getbootvolgs(var pblockptr: getnamerecgs);
      inline $2028F4, $E100B022, $8F, '_toolErr';

    procedure getdevnumbergs(var pblockptr: devnumrecgs);
      inline $2020F4, $E100B022, $8F, '_toolErr';

    procedure getdirentrygs(var pblockptr: direntryrecgs);
      inline $201CF4, $E100B022, $8F, '_toolErr';

    procedure geteofgs(var pblockptr: eofrecgs);
      inline $2019F4, $E100B022, $8F, '_toolErr';

    procedure getfileinfogs(var pblockptr: fileinforecgs);
      inline $2006F4, $E100B022, $8F, '_toolErr';

    procedure getfstinfogs(var pblockptr: fstinforecgs);
      inline $202BF4, $E100B022, $8F, '_toolErr';

    procedure getlevelgs(var pblockptr: levelrecgs);
      inline $201BF4, $E100B022, $8F, '_toolErr';

    procedure getmarkgs(var pblockptr: positionrecgs);
      inline $2017F4, $E100B022, $8F, '_toolErr';

    procedure getnamegs(var pblockptr: getnamerecgs);
      inline $2027F4, $E100B022, $8F, '_toolErr';

    procedure getprefixgs(var pblockptr: prefixrecgs);
      inline $200AF4, $E100B022, $8F, '_toolErr';

    procedure getrefinfogs(var pblockptr: refinforecgs);
      inline $2039F4, $E100B022, $8F, '_toolErr';

    procedure getrefnumgs(var pblockptr: getrefnumrecgs);
      inline $2038F4, $E100B022, $8F, '_toolErr';

    procedure getstdrefnumgs(var pblockptr: stdrefnumrecgs);
      inline $2037F4, $E100B022, $8F, '_toolErr';

    procedure getsysprefsgs(var pblockptr: sysprefsrecgs);
      inline $200FF4, $E100B022, $8F, '_toolErr';

    procedure getversiongs(var pblockptr: versionrecgs);
      inline $202AF4, $E100B022, $8F, '_toolErr';

	procedure judgenamegs(var pblockptr: judgenamerecgs);
	  inline $F4, $2007, $22, $E100B0, $8F, '_toolErr';

    procedure newlinegs(var pblockptr: newlinerecgs);
      inline $2011F4, $E100B022, $8F, '_toolErr';

    procedure nullgs(var pblockptr: intptr);
      inline $200DF4, $E100B022, $8F, '_toolErr';

    procedure opengs(var pblockptr: openrecgs);
      inline $2010F4, $E100B022, $8F, '_toolErr';

    procedure osshutdowngs(var pblockptr: osshutdownrecgs);
      inline $2003F4, $E100B022, $8F, '_toolErr';

    procedure quitgs(var pblockptr: quitrecgs);
      inline $2029F4, $E100B022, $8F, '_toolErr';

    procedure readgs(var pblockptr: iorecgs);
      inline $2012F4, $E100B022, $8F, '_toolErr';

    procedure resetcachegs(var pblockptr: intptr);
      inline $2026F4, $E100B022, $8F, '_toolErr';

    procedure sessionstatusgs(var pblockptr: sessionstatusrecgs);
      inline $201FF4, $E100B022, $8F, '_toolErr';

    procedure seteofgs(var pblockptr: setpositionrecgs);
      inline $2018F4, $E100B022, $8F, '_toolErr';

    procedure setfileinfogs(var pblockptr: fileinforecgs);
      inline $2005F4, $E100B022, $8F, '_toolErr';

    procedure setlevelgs(var pblockptr: levelrecgs);
      inline $201AF4, $E100B022, $8F, '_toolErr';

    procedure setmarkgs(var pblockptr: setpositionrecgs);
      inline $2016F4, $E100B022, $8F, '_toolErr';

    procedure setprefixgs(var pblockptr: prefixrecgs);
      inline $2009F4, $E100B022, $8F, '_toolErr';

    procedure setstdrefnumgs(var pblockptr: stdrefnumrecgs);
      inline $203AF4, $E100B022, $8F, '_toolErr';

    procedure setsysprefsgs(var pblockptr: sysprefsrecgs);
      inline $200CF4, $E100B022, $8F, '_toolErr';

    procedure unbindintgs(var pblockptr: interruptrecgs);
      inline $2032F4, $E100B022, $8F, '_toolErr';

    procedure volumegs(var pblockptr: volumerecgs);
      inline $2008F4, $E100B022, $8F, '_toolErr';

    procedure writegs(var pblockptr: iorecgs);
      inline $2013F4, $E100B022, $8F, '_toolErr';

  implementation
end.
