{********************************************
; File: Shell.p
;
;
; Copyright Apple Computer, Inc. 1986-92
; All Rights Reserved
;
********************************************}

UNIT SHELL;
INTERFACE
USES TYPES, GSOS;

TYPE
GetLInfoPBPtr = ^GetLInfoPB;
GetLInfoPB = PACKED RECORD
    sfile : StringPtr; { address of source file name }
    dfile : StringPtr; { address of output file name }
    parms : StringPtr; { address of parameter list }
    istring : StringPtr; { address of language specific input string }
    merr : Byte; { max error level allowed }
    merrf : Byte; { max error level found }
    lops : Byte; { operations flag }
    kflag : Byte; { KEEP flag }
    mflags : Longint; { set of letters selected with '-' }
    pflags : Longint; { set of letters selected with '+' }
    org : Longint; { abs start address of non-relloc load file }
END;

{ GetLInfo/SetLInfo parameter block for level 1 calls }
GetLInfoGSPBPtr = ^GetLInfoGSPB;
GetLInfoGSPB = PACKED RECORD
	pCount	:	Integer;			{ parameter count }
	sfile	:	StringPtr;			{ address of source file name }
	dfile	:	StringPtr;			{ address of output file name }
	parms	:	StringPtr;			{ address of paramter list }
	istring	:	StringPtr;			{ address of language specific input string }
	merr	:	Byte;				{ max error level allowed }
	merrf	:	Byte;				{ max error level found }
	lops	:	Byte;				{ operations flag }
	kflag	:	Byte;				{ KEEP flag }
	mflags	:	LongInt;			{ set of letters selected with '-' }
	pflags	:	LongInt;			{ set of letters selected with '+' }
	org		:	LongInt;			{ abs start address of non-relloc load file }
END;

{ FastFile call parameter block for level 0 calls }
FastFilePB = PACKED RECORD
	action	:	Integer;			{ action parameter }
	index	:	Integer;			{ index parameter }
	flags	:	Integer;			{ flags parameter }
	file_handle : Handle;			{ handle to contents of file }
	file_length : LongInt;			{ length of file in memory }
	name	:	StringPtr;			{ name of file }
	access	:	Integer;			{ access attribute }
	file_type :	Integer;			{ file type of file }
	aux_type :	LongInt;			{ aux type of file }
	storage_type : Integer;			{ storage type of file }
	create_date : Integer;			{ creation date of file }
	create_time : Integer;			{ creation time of file }
	mod_date	: Integer;			{ modification date of file }
	mod_time	: Integer;			{ modification time of file }
	blocks_used : LongInt;			{ blocks of disk space used by file }
END;

{ FastFile call parameter block for level 1 calls }
FastFileGSPB = PACKED RECORD
	pCount	:	Integer;			{ parameter count }
	action	:	Integer;			{ action parameter }
	index	:	Integer;			{ index parameter }
	flags	:	Integer;			{ flags parameter }
	fileHandle : Handle;			{ handle to contents of file }
	pathName :	Ptr;
	access	:	Integer;			{ access attribute }
	fileType :	Integer;			{ file type of file }
	auxType	:	LongInt;			{ aux type of file }
	storageType : Integer;			{ storage type of file }
	createDate : TimeRec;			{ creation date and time }
	modDate	:	TimeRec;			{ modification date and time }
	option : ResultBuf255Ptr;		{ option list }
	fileLength : LongInt;			{ length of file in memory }
	blocksUsed : LongInt;			{ blocks of disk space used by file }
END;

{ parameter block for GetIODevices and SetIODevices calls }
IODevPBPtr = ^IODevPB;
IODevPB = PACKED RECORD
	output_type	:	Integer;
	output_addr	:	ProcPtr;
	error_type	:	Integer;
	error_addr	:	ProcPtr;
	input_type	:	Integer;
	input_addr	:	ProcPtr;
END;

IODevGSPBPtr = ^IODevGSPB;
IODevGSPB = PACKED RECORD
	pCount		:	Integer;
	outputType	:	Integer;
	outputAddr	:	ProcPtr;
	errorType	:	Integer;
	errorAddr	:	ProcPtr;
	inputType	:	Integer;
	inputAddr	:	ProcPtr;
END;

{ GetCommand call parameter block }
GetCommandPB = PACKED RECORD
	index	:	Integer;
	restart	:	Integer;
	__reserved__ : Integer;
	command	:	Integer;
	name	:	Array[0..15] of Char;
END;

GetCommandGSPBPtr = ^GetCommandGSPB;
GetCommandGSPB = PACKED RECORD
	pCount	:	Integer;
	index	:	Integer;
	restart	:	Integer;
	__reserved__ : Integer;
	command	:	Integer;
	name	:	String[15];
END;

{ ChangeVector call parameter block }
ChangeVectorPB = PACKED RECORD
	__reserved__ :	Integer;
	vector		:	Integer;
	procPtr		:	ProcPtr;
	oldProcPtr	:	ProcPtr;
END;

ChangeVectorGSPB = PACKED RECORD
	pCount		:	Integer;
	__reserved__ :	Integer;
	vector		:	Integer;
	procPtr		:	ProcPtr;
	oldProcPtr	:	ProcPtr;
END;

{ parameter block for level 1 NextWildcard call }
NextWildcardGSPB = PACKED RECORD
	pCount		:	Integer;
	pathName	:	ResultBuf255Ptr;
	access		:	Integer;
	fileType	:	Integer;
	auxType		:	LongInt;
	storageType	:	Integer;
	createDate	:	TimeRec;
	modDate		:	TimeRec;
	option		:	Ptr;
	EOF			:	LongInt;
	blocksUsed	:	LongInt;
	resourceEOF	:	LongInt;
	resourceBlocks : LongInt;
END;

Procedure CHANGE_VECTOR ( var pb : ChangeVectorPB ) ; C;
Procedure CONSOLE_OUT ( oChar : Char ) ; C;
PROCEDURE DIRECTION ( device:Integer; var direct: Integer)  ; C;
PROCEDURE ERROR ( errorCode:Integer)  ; C;
PROCEDURE EXECUTE ( flag:Integer; comm:Ptr)  ; C;
Procedure EXPAND_DEVICES( name : StringPtr ); C;
Procedure EXPORT(variable : StringPtr; flag : Integer); C;
Procedure FASTFILE(var pb : FastFilePB); C;
Procedure GET_COMMAND(var pb : GetCommandPB); C;
Procedure GET_IO_DEVICES( pb : IODevPBPtr); C;
FUNCTION  GET_LANG  : Integer ; C;
PROCEDURE GET_LINFO ( PblockPtr:GetLInfoPBPtr)  ; C;
PROCEDURE GET_VAR ( varname:StringPtr; value:StringPtr)  ; C;
PROCEDURE INIT_WILDCARD ( initFile:StringPtr; flags:Integer)  ; C;
PROCEDURE NEXT_WILDCARD ( nextFile:StringPtr); C;
Procedure POP_VARIABLES;
	Inline	$22, $E100A8, $0117, $0, $0, $0, $0;
Procedure PUSH_VARIABLES;
	Inline	$22, $E100A8, $0118, $0, $0, $0, $0, $8F, '_toolErr';
PROCEDURE READ_INDEXED ( varname:StringPtr; value:StringPtr; index:Integer)  ; C;
Procedure READ_VARIABLE ( varname:StringPtr; value:StringPtr); C;
PROCEDURE REDIRECT ( device:Integer; app:Integer; theFile:StringPtr)  ; C;
Procedure SET_IO_DEVICES(pb : IODevPBPtr); C;
PROCEDURE SET_LANG ( language:Integer)  ; C;
PROCEDURE SET_LINFO ( PblockPtr:GetLInfoPBPtr)  ; C;
Procedure SET_STOP_FLAG(flag : Integer); C;
PROCEDURE SET_VAR ( varname:StringPtr; value:StringPtr)  ; C;
FUNCTION  STOP  : Integer ; C;
Procedure UNSET_VARIABLE( name : StringPtr ); C;
FUNCTION  VERSION  : Longint ; C;
PROCEDURE WRITE_CONSOLE ( ochar:Char )  ; C;

{	New ORCA 2.0 level 1 calls				}

Procedure ChangeVectorGS( var pb : ChangeVectorGSPB); C;
Procedure ConsoleOutGS( oChar : Char ); C;
Function DirectionGS(device : Integer) : Integer; C;
Procedure ErrorGS( errorCode : Integer ); C;
Procedure ExecuteGS( flag : Integer; comm : Ptr); C;
Function ExpandDevicesGS(name : StringPtr) : StringPtr;
Procedure ExportGS(name : StringPtr; flags : Integer);
Procedure FastFileGS(var pb : FastFileGSPB); C;
Function GetCommandGS( index : Integer) : GetCommandGSPBPtr;
Function GetIODevicesGS : IODevGSPBPtr;
Function GetLangGS : Integer; C;
Function GetLInfoGS : GetLInfoGSPBPtr;
Function GetVarGS( variable : StringPtr) : StringPtr;
Procedure InitWildcardGS(name : StringPtr; flags : Integer);
Function NextWildcardGS : StringPtr;
Procedure PopVariablesGS;
Procedure PushVariablesGS;
Procedure ReadIndexedGS(var name, value : StringPtr; index : Integer);
Function ReadVariableGS(name : StringPtr) : StringPtr;
Procedure RedirectGS(device, app : Integer; theFile : StringPtr);
Procedure SetIODevicesGS(parms : IODevGSPBPtr); C;
Procedure SetLangGS(language : Integer); C;
Procedure SetLInfoGS(parmsPtr : GetLInfoGSPBPtr);
Procedure SetStopFlagGS(flag : Integer); C;
Procedure SetVarGS(name, value : StringPtr; exportFlag : Integer);
Function StopGS : Integer; C;
Procedure UnsetVariableGS(name : StringPtr);
Function VersionGS : LongInt; C;
Procedure WriteConsoleGS( oChar : Char); C;

IMPLEMENTATION
END.
