{---------------------------------------------------------------------

  file ListWindow.p
  
  Part of HyperCard IIGS 1.1

  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
  ListWindow windowTitle, listWidth, listHeight, listMessage, listContents
	
	INPUTS:  windowTitle	Text displayed above the list control
			 listWidth	  	Width of the list control in 320 pixels
			 listHeight	  	Height of the list control in lines
			 listMessage	Message to send to HyperCard when one of the
							items in the list is double-clicked.  Within
							this string, the string <listItem> will be
							replaced with the item clicked.
								  
							EXAMPLE:
							listMessage:  open stack "<listItem>"
								  
							If the user clicks an item called "MyStack",
							the message:  open stack "MyStack" is sent 
							starting at the card level.
							
			 doSort			Boolean value indicating whether to sort the 
			 				list.
			 listContents  	Return delimited list of items to display in
							the list control.
								  

  This XCMD creates and maintains an XWindow.  The XWindow contains a list
  control from which the user can select an item which causes a specific
  message to be sent.
  
  
  Author:	Darin Acquistapace
  Created:	7/26/91
  Modified:	See Mod History Below


  Modification History:

	1/29/92 - Released today as sample code.
	
---------------------------------------------------------------------}

UNIT DummyUnit;

{$Z+ }
{$N+ }

INTERFACE

USES 	
	Types, Memory, MiscTool, GSOS, QuickDraw, Resources, QDAux, Events, 
	Controls, Windows, Lists, HyperXCMD;
    
PROCEDURE EntryPoint(paramPtr: XCMDPtr);

IMPLEMENTATION

PROCEDURE ListWindow(paramPtr: XCMDPtr);									FORWARD;

PROCEDURE EntryPoint(paramPtr: XCMDPtr);
BEGIN
  ListWindow(paramPtr);
END;

FUNCTION CountLines(sourceString: Ptr): INTEGER; 							EXTERNAL;

PROCEDURE ListWindow(paramPtr: XCMDPtr);
CONST
	LeftPos 		= 50;
	TopPos			= 75;
	MinWidth 		= 10;
	MaxWidth		= 280;
	MinHeight 		= 40;
	MaxHeight		= 250;
	ScriptErrStr	= 'Can''t understand arguments of XCMD ListWindow.';
	CopyrightStr	= 'ListWindow XCMD v1.0.1" & return & "by Darin Acquistapace, 7/26/91" & return & " 1991 Apple Computer, Inc.';
	HelpStr			= 'FORM:  ListWindow windowTitle, listWidth," & return & "           numLines, listMessage, doSort," & return & "           listContents"';
	WrongVersionStr = 'ListWindow XCMD requires HyperCard IIGS 1.1';
	CreateErrStr	= 'Unable to create window';
	OutOfMemoryStr	= 'Not enough memory to create XWindow';

TYPE
    ListWindHndl	= ^ListWindPtr;
	ListWindPtr		= ^ListWindRec;
	ListWindRec		= RECORD
	  saveListTextHandle: 	Handle;
	  saveMemHandle:		Handle;
	  saveTitleHandle:		Handle;
	  saveMsgHandle:		Handle;
	END; {record}
	MemRecArrayPtr = ^MemRecArray;
	MemRecArray = PACKED ARRAY[0..0] OF MemRec;
	
VAR
	listWidth:		INTEGER;
	listHeight:		INTEGER;
	doSort:			BOOLEAN;
	xWindow:		WindowPtr;
	sizeCtlHndl:	CtlRecHndl;
	windRect:		Rect;
	str:			Str255;
	title:			Str255;
	msg:			Str255;
	
	
	PROCEDURE MyDisposeHandle(h: Handle);
	{ Checks a handle for NIL before disposing of it }
	BEGIN
	  IF h <> NIL THEN DisposeHandle(h);
	END; {MyDisposeHandle}
	
	
	FUNCTION PinWord(targetValue, minValue, maxValue: INTEGER): INTEGER;
	{ Keeps the specified integer within bounds }
	BEGIN
	  IF targetValue < minValue THEN targetValue := minValue ELSE
	    IF targetValue > maxValue THEN targetValue := maxValue;
	  PinWord := targetValue;
	END; {PinWord}
	
	
	PROCEDURE HTError;
	{ Puts the specified string into the result and terminates with a HyperTalk error }
	BEGIN
	  paramPtr^.returnValue := PasToZero(ScriptErrStr);
	  paramPtr^.returnStat := 1;
	  EXIT(ListWindow);
	END; {HTError}
	
	
	PROCEDURE ReturnResult(str: Str255);
	{ Puts the specified string into the result and terminates }
	BEGIN
	  paramPtr^.returnValue := PasToZero(str);
	  EXIT(ListWindow);
	END; {ReturnResult}
	
	
	PROCEDURE CommandInfo(msg : str255);
	{ Puts the specified string into a dialog box and terminates }
	BEGIN
	  SendHCmessage(Concat('answer "', msg, '"')); 
	  EXIT(ListWindow);
	END; {CommandInfo}


	PROCEDURE UpdateListWindow(whichWindow: WindowPtr);
	{ This draws our picture into the XWindow }
	BEGIN
	  DrawControls(whichWindow);
	END; {UpdatePicture}


	PROCEDURE CleanUpMemory(whichWindow: WindowPtr);
	{ Free all memory associated with the XWindow }
	VAR
	  saveWindInfo:	ListWindHndl;
	BEGIN
	  saveWindInfo := ListWindHndl(GetXWindowValue(whichWindow));
	  WITH saveWindInfo^^ DO BEGIN
	    MyDisposeHandle(Handle(saveListTextHandle));
	    MyDisposeHandle(Handle(saveMemHandle));
	    MyDisposeHandle(Handle(saveTitleHandle));
	    MyDisposeHandle(Handle(saveMsgHandle));
	  END; {WITH}
	  MyDisposeHandle(Handle(saveWindInfo));
	END; {CleanUpMemory}
	

	PROCEDURE DoSendCardMessage(whichWindow: WindowPtr; myListControl: CtlRecHndl);
	{ Here is where we combine the specified message with the item clicked and send
	  it as a card message }
	CONST
	  targetText		= '<listitem>';
	TYPE
	  Str255Hndl		= ^Str255Ptr;
	  Str255Ptr			= ^Str255;
	VAR
	  offset:			LongInt;
	  index:			INTEGER;
	  selectedNum:		INTEGER;
	  saveWindInfo:		ListWindHndl;
	  startPtr:			Ptr;
	  offsetPtr:		Ptr;
	  str1:				Str255;
	  str2:				Str255;
	  str3:				Str255;
	BEGIN
	  saveWindInfo := ListWindHndl(GetXWindowValue(whichWindow));
	  HLock(saveWindInfo^^.saveMsgHandle);
	  startPtr := saveWindInfo^^.saveMsgHandle^;
	  BlockMove(startPtr, @str1, 255);
	  IF LENGTH(str1) = 255 THEN
	    str1[255] := CHR(0)
	  ELSE
	    str1 := Concat(str1, CHR(0));
	  offsetPtr := StringMatch(targetText, @str1);
	  IF offsetPtr = NIL THEN
	    SendCardMessage(Str255Hndl(saveWindInfo^^.saveMsgHandle)^^)
	  ELSE BEGIN
	    offset := ORD(offsetPtr)-ORD(@str1);
		str2 := '';
		FOR index := 1 to offset-1 DO
		  str2 := Concat(str2, str1[index]);

		str3 := '';
		selectedNum := NextMember2(0, myListControl);
		IF selectedNum > 0 THEN
		  ZeroToPas(MemRecArrayPtr(saveWindInfo^^.saveMemHandle^)^[selectedNum-1].memPtr, str3);
		str2 := Concat(str2, str3);

		FOR index := offset + 10 to LENGTH(str1)-1 DO
		  str2 := Concat(str2, str1[index]);
		
	    SendHCMessage(str2);
	  END; {if}
	  HUnlock(saveWindInfo^^.saveMsgHandle);
	END; {DoSendCardMessage}
	
	
	PROCEDURE HandleWindowClick(whichWindow: WindowPtr; ourEvent: EventRecord);
	{ The user has clicked in our window, handle tracking the click.  This
	  includes letting the list manager deal with clicks in the list control as
	  well as manually determining whether the user has double-clicked an item. 
	  
	  NOTE:  We only have one active control in our window so we assume the
	      	 found control is the list control. }
	VAR
	  selectedPart:	INTEGER;
	  dummyResult:	INTEGER;
	  dblClickTime:	INTEGER;
	  curTick:		LongInt;
	  foundCtl:		CtlRecHndl;
	  initialLoc:	Point;
	BEGIN
	  IF FindControl(foundCtl, ourEvent.where.h, ourEvent.where.v, 
	  				 whichWindow) <> 0 THEN BEGIN
		dblClickTime := GetDblTime;  { Get the amount of time to wait for a dbl click }
		initialLoc := ourEvent.where;
	    selectedPart := TrackControl(initialLoc.h, initialLoc.v, Pointer(-1), foundCtl);
		IF selectedPart = 1 THEN BEGIN
		  curTick := GetTick;
		  REPEAT
		    IF EventAvail(mDownMask, ourEvent) THEN BEGIN  { Get mouseDown }
			  IF (Abs(ourEvent.where.h - initialLoc.h) < 8) &
			     (Abs(ourEvent.where.v - initialLoc.v) < 4) THEN BEGIN
				REPEAT UNTIL EventAvail(mUpMask, ourEvent); { Wait for mouseUp }
			    dummyResult := FlushEvents(mDownMask + mUpMask, 0);
			    DoSendCardMessage(whichWindow, foundCtl);
			    LEAVE;
			  END; {if}
			END; {if}
		  UNTIL GetTick > curTick + dblClickTime;
		END; {if}
	  END; {if}
	END; {HandleWindowClick}
	

	PROCEDURE HandleEvents;
	{ Handle events specific to our XWindow }
	VAR
	  myEventInfo:	XWEventInfoPtr;
	  window:		WindowPtr;
	  event:		EventRecord;
	BEGIN
      myEventInfo := XWEventInfoPtr(paramPtr^.params[1]);
      window := myEventInfo^.eventWindow;
      event := myEventInfo^.event;
	  
	  CASE event.what OF
	    xOpenEvt:			XWAllowReEntrancy(window, TRUE, TRUE);
	    updateEvt:			UpdateListWindow(window);
		mouseDownEvt:		HandleWindowClick(window, event);
  		xCloseEvt:			CleanUpMemory(window);
		xCursorWithin:		paramPtr^.passFlag := TRUE;		{ Let HC change our cursor }
	  END; {case}
	END; {HandleEvents}


	PROCEDURE SetUpContents(whichWindow: WindowPtr; title: Str255;
							listWidth, listHeight: INTEGER; msg: Str255;
							doSort: BOOLEAN; itemList: Handle);
	{ Create the contents of our window.  This includes one static text
	  item and one list control. }
	TYPE
	  ListPtr = ^MyListRec;
	  MyListRec = PACKED RECORD
		pCount: 		INTEGER;
		controlID: 		LongInt;
		controlRect: 	Rect;
		procRef: 		LongInt;
		flag : 			INTEGER;
		moreFlags: 		INTEGER;
		refCon: 		Ptr;
		listSize: 		INTEGER;
		listView: 		INTEGER;
		listType: 		INTEGER;
		listStart: 		INTEGER;
		listDraw: 		Ptr;
		listMemHeight:  INTEGER;
		listMemSize: 	INTEGER;
		listRef: 		Ptr;
	  END; {record}
	  MyStatTextRecord = PACKED RECORD
		pCount: 		INTEGER;
		id: 			LongInt;
		textRect: 		Rect;
		procRef: 		LongInt;
		flag: 			INTEGER;
		moreFlags: 		INTEGER;
		refCon: 		LongInt;
		textRef: 		Ptr;
		textSize: 		INTEGER;
	  END; {record}
	
	VAR
	  numberLines, myID: 				INTEGER;
	  memHandle, listTextHandle,
	  titleHandle, msgHandle: 			Handle;
	  saveWindInfo:						ListWindHndl;
	  listTextSize:						LongInt;
	  listHandle, statControl: 			CtlRecHndl;
	  theRect:							Rect;
	  myFieldListPtr: 					MemRecArrayPtr;
	  listRecord: 						MyListRec;
	  myText: 							MyStatTextRecord;


	PROCEDURE ExpandFieldText(myTextLoc: Ptr);
	{ This puts zeros into all the spots in the list text where returns used 
	  to be.  This way, I can pass a copy of the parameter list directly to the 
	  list manager as a series of C strings }
	VAR
	  workChar: 			Byte;
	  totalLines: 			INTEGER;
	  startPtr: 			Ptr;
	BEGIN
	  startPtr := myTextLoc;
	  numberLines := 0;
	  { Count the number of lines so I know how big to make my handle which contains 
	    the ptrs to the individual text lines }
	  totalLines := CountLines(listTextHandle^);
	  IF totalLines = 0 THEN BEGIN
		numberLines := 0;
		memHandle := NIL;
		EXIT(ExpandFieldText);
	  END; {if}
	  memHandle := NewHandle(totalLines*SIZEOF(MemRec), myID, attrLocked+attrFixed, NIL);
	  IF MemHandle = NIL THEN ReturnResult(OutOfMemoryStr);
	  myFieldListPtr := MemRecArrayPtr(memHandle^);
	  REPEAT
		workChar := myTextLoc^[1];
		IF (workChar = 13) OR (workChar = 0) THEN BEGIN
		  myFieldListPtr^[numberLines].memPtr := startPtr;
		  myFieldListPtr^[numberLines].memFlag := 0;
		  myTextLoc^[1] := 0;
		  startPtr := Pointer(ORD4(myTextLoc)+1);
		  numberLines := numberLines + 1;
		end; {if}
		myTextLoc := Pointer(ORD4(myTextLoc)+1);
	  UNTIL WorkChar = 0;
	END; {ExpandFieldText}
	
	
	BEGIN
	  myID := paramPtr^.UserID;
	  listTextSize := GetHandleSize(itemList);
	  IF listTextSize = 0 THEN listTextSize := 1;
	
	  listTextHandle := NewHandle(listTextSize, myID, attrFixed+attrLocked, NIL);
	  IF (_toolErr <> noError) OR (listTextHandle = NIL) THEN BEGIN
		CloseXWindow(whichWindow);
	    ReturnResult(OutOfMemoryStr);
	  END; {if}
	  BlockMove(itemList^, listTextHandle^, listTextSize);
	
	  { Save the title of the window }
	  titleHandle := NewHandle(LENGTH(title)+1, myID, attrFixed+attrLocked, NIL);
	  IF (_toolErr <> noError) OR (titleHandle = NIL) THEN BEGIN
		CloseXWindow(whichWindow);
	    MyDisposeHandle(listTextHandle);
	    ReturnResult(OutOfMemoryStr);
	  END; {if}
	  BlockMove(@title, titleHandle^, LENGTH(title)+1);
	
	  { Save the message to be sent }
	  msgHandle := NewHandle(LENGTH(msg)+1, myID, attrNoPurge, NIL);
	  IF (_toolErr <> noError) OR (msgHandle = NIL) THEN BEGIN
		CloseXWindow(whichWindow);
	    MyDisposeHandle(titleHandle);
	    MyDisposeHandle(listTextHandle);
	    ReturnResult(OutOfMemoryStr);
	  END; {if}
	  BlockMove(@msg, msgHandle^, LENGTH(msg)+1);
	
	  { Create a record within a handle we will use to store the info we need about
	    this window until we close it }
	  saveWindInfo := ListWindHndl(NewHandle(SIZEOF(ListWindRec), myID, 0, NIL));
	  
	  ExpandFieldText(listTextHandle^);

	  { Create a record containing info we'll need later }
	  WITH saveWindInfo^^ DO BEGIN
	    saveListTextHandle := listTextHandle;
		saveMemHandle := memHandle;
		saveTitleHandle := titleHandle;
		saveMsgHandle := msgHandle;
	  END; {with}
	  
	  { Save the handle to our needed info so we can access it in response to events }
	  SetXWindowValue(whichWindow, LongInt(saveWindInfo));

	  { Create the static text item }
	  WITH MyText DO BEGIN
		pCount := 8;
		id := 6;
		WITH textRect DO BEGIN
		  left := 10; top := 1; bottom := 11;
		  right := whichWindow^.portRect.right - whichWindow^.portRect.left - 10;
		END; {with}
		procRef := statTextControl;
		flag := 1;
		moreFlags := fCtlProcRefNotPtr;
		refCon := 0;
		textRef := Pointer(ORD(titleHandle^)+1);
		textSize := LENGTH(title);
	  END; {with}
	  statControl := NewControl2(whichWindow, 0, ref(@MyText));
	
	  { Create the List Field }
	  WITH listRecord DO BEGIN
		pCount := 14;
		controlID := 5;
		WITH controlRect DO BEGIN
			left := -2; top := 11; right := listWidth+2; bottom := listHeight+12;
		END; {with}
		procRef := ListControl;
		flag := 0;
		moreFlags := fCtlProcRefNotPtr + fCtlIsMultiPart;
		refCon := NIL;
		listSize := NumberLines;
		listView := listHeight DIV 10;
		listType := 7;
		listStart := 1;
		listDraw := NIL;
		listMemHeight := 10;
		listMemSize := SIZEOF(MemRec);
		listRef := Pointer(myFieldListPtr);
	  END; {with}
	  listHandle := NewControl2(whichWindow, 0, REF(@listRecord));
	  IF doSort THEN SortList2(NIL, listHandle);
	  IF numberLines > 0 THEN SelectMember2(1, listHandle);
	END; {SetupContents}
	
	
	FUNCTION CorrectVersion: BOOLEAN;
	{ This returns true if the version of HyperCard is >= minVersion }
	CONST
	  minVersion = '1.1';
	VAR
	  tempHandle:	Handle;
	  tempStr:		Str255;
	BEGIN
	  tempHandle := EvalExpr('the version');
	  ZeroToPas(tempHandle^, tempStr);
	  MyDisposeHandle(tempHandle);
	  CorrectVersion := tempStr >= minVersion;
	END; {CorrectVersion}
	

  BEGIN
	{ If the paramCount is negative, we have been called in response to an event }
	IF paramPtr^.paramCount < 0 THEN BEGIN
	  HandleEvents;
	  EXIT(ListWindow);
	END; {if}

	{ Display help or copyright info in response to "?" or "!" }
	IF paramPtr^.paramCount = 1 THEN BEGIN
	  ZeroToPas(paramPtr^.params[1]^, str);
	  IF str = '!' THEN CommandInfo(CopyrightStr);
	  IF str = '?' THEN CommandInfo(HelpStr);
	END; {if}
	
	{ Make sure we are running at least version 1.1 of HyperCard IIGS }
    IF NOT CorrectVersion THEN ReturnResult(WrongVersionStr);
	
	{ We need exactly six parameters }
	IF paramPtr^.paramCount <> 6 THEN HTError;

	{ Grab the user parameters }
	ZeroToPas(paramPtr^.params[1]^, title);
	ZeroToPas(paramPtr^.params[2]^, str);
	listWidth := PinWord(StrToNum(str)*2, MinWidth, MaxWidth);
	ZeroToPas(paramPtr^.params[3]^, str);
	listHeight := PinWord(StrToNum(str)*10, MinHeight, MaxHeight);
	ZeroToPas(paramPtr^.params[4]^, msg);
	ZeroToPas(paramPtr^.params[5]^, str);
	doSort := StringEqual(str, 'true');
	
	SetRect(windRect, LeftPos, TopPos, LeftPos+listWidth, TopPos+listHeight+11);
	xWindow := NewXWindow(windRect, title, FALSE, xWindoidStyle);   	{ Create window invisible }		
    IF xWindow = NIL THEN ReturnResult(CreateErrStr);
	
	{ Create the contents for the window }
	SetPort(xWindow);
	SetUpContents(xWindow, title, listWidth, listHeight, msg, doSort, 
				  paramPtr^.params[6]);
    ShowWindow(xWindow);								{ and show it }
  END; {ListWindow}

END.	{ of the dummy unit	}