{---------------------------------------------------------------------

  file MediaControl.inc
  
  Part of HyperCard IIGS 1.1

  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
	MediaControl channelNum, command
	
  This XCMD provides an interface to the Media Control Toolset
  (Tool 38) from HyperTalk.  It also enables or disables the
  video overlay feature of the Apple II Video Overlay card.
  
  NOTE:  This is not the complete MediaControl interface file
  		 for the MediaControl tool but only interfaces for
		 those routines actually used in the MediaControl XCMD.
		 
  Author:	Darin Acquistapace
  Created:	6/25/91
  Modified:	See Mod History Below


  Modification History:

	1/28/91 - Release today as sample code.
	
---------------------------------------------------------------------}


{ Control values for MCControl }
CONST
	mcCInit			=	$0001;		{ Initialize player. }
	mcCEject		=	$0002;		{ Eject disc. }
	mcCVideoOn		=	$0003;		{ Turn video on. }
	mcVideoOff		=	$0004;		{ Turn video off. }
	mcCDisplayOn	=	$0005;		{ Turn video position display on. }
	mcCDisplayOff	=	$0006;		{ Turn video position display off. }
	mcCBlankVideo	=	$0007;		{ Blank video for next MCSearchTo. }
	mcCDefaultCom	=	$0008;		{ Set default communications. }
	mcCLockDev		=	$0009;		{ Set the device to locked. }
	mcCUnLockDev	=	$000A;		{ Unlock the device. }

	mcC8Data1Stop	=	$0028;		{ Set 8 - data  1 - stop bit. }
	mcC7Data1Stop	=	$0029;		{ Set 7 - data  1 - stop bit. }
	mcC6Data1Stop	=	$002A;		{ Set 6 - data  1 - stop bit. }
	mcC5Data1Stop	=	$002B;		{ Set 5 - data  1 - stop bit. }
	mcC8Data2Stop	=	$002C;		{ Set 8 - data  2 - stop bit. }
	mcC7Data2Stop	=	$002D;		{ Set 7 - data  2 - stop bit. }
	mcC6Data2Stop	=	$002E;		{ Set 6 - data  2 - stop bit. }
	mcC5Data2Stop	=	$002F;		{ Set 5 - data  2 - stop bit. }

	mcCBaudDflt		=	$0032;		{ Set baud rate to control panel setting. }
	mcCBaud50		=	$0033;		{ Set 50 baud. }
	mcCBaud75		=	$0034;		{ Set 75 baud. }
	mcCBaud110		=	$0035;		{ Set 110 baud. }
	mcCBaud134		=	$0036;		{ Set 134 baud. }
	mcCBaud150		=	$0037;		{ Set 150 baud. }
	mcCBaud300		=	$0038;		{ Set 300 baud. }
	mcCBaud600		=	$0039;		{ Set 600 baud. }
	mcCBaud1200		=	$003A;		{ Set 1200 baud. }
	mcCBaud1800		=	$003B;		{ Set 1800 baud. }
	mcCBaud2400		=	$003C;		{ Set 2400 baud. }
	mcCBaud3600		=	$003D;		{ Set 3600 baud. }
	mcCBaud4800		=	$003E;		{ Set 4800 baud. }
	mcCBaud7200		=	$003F;		{ Set 7200 baud. }
	mcCBaud9600		=	$0040;		{ Set 9600 baud. }
	mcCBaud19200	=	$0041;		{ Set 19200 baud. }

	mcCModem		=	$0064;		{ Set to modem port. }
	mcCPrinter		=	$0065;		{ Set to printer port. }

	mcCIgnoreDS		=	$00C8;		{ Ignore "disk switched" GSOS errors. }
	mcCReportDS		=	$00C9;		{ Report "disk switched" GSOS errors. }


{ Values for MCGetFeatures }

	mcFTypes		=	0;			{ How the disc is organized (frames, chapters, etc.). }
	mcFStep			=	1;			{ Maximum fps speed value (normal is 255). }
	mcFRecord		=	2;			{ Device supports MCRecord. }
	mcFVideo		=	3;			{ Device supports toggling video. }
	mcFEject		=	4;			{ Device supports ejecting medium. }
	mcFLock			=	5;			{ Device supports user lock (locking user from 
								  	  physically operating the device). }
	mcFVDisplay		=	6;			{ Device supports video display of location. }
	mcFVOverlay		=	7;			{ No. of lines of overlay characters device supports. }
	mcFVOChars		=	8;			{ No. of chars/line supported by overlay. }


{ Position unit types for MCGetPosition and other calls }

	mcInChapters	=	1;			{ Position is measured in chapters. }
	mcInFrames		=	2;			{ Position is measured in frames. }
	mcInTime		=	3;			{ Position is measured in hours,minutes,seconds,blocks. }


{ Status values for MCGetStatus }

	mcSUnknown		=	$0000;		{ Player unable to determine this status. }

	mcSDeviceType	=	$0000;		{ "Device type" selector. }
	mcSLaserDisc	=	$0001;		{ Video laser disc player. }
	mcSCDAudio		=	$0002;		{ Audio CD player. }
	mcSLaserCD		=	$0003;		{ Combination laser/CD player. }
	mcSVCR			=	$0004;		{ VCR. }
	mcSCamCorder	=	$0005;		{ Video camera. }

	mcSPlayStatus	=	$0001;		{ "Play status" selector. }
	mcSPlaying		=	$0001;		{ The device is playing. }
	mcSStill		=	$0002;		{ The device is not playing (paused). }
	mcSParked		=	$0003;		{ The device is shut down. }

	mcSDoorStatus	=	$0002;		{ "Player door" status. }
	mcSDoorOpen		=	$0001;
	mcSDoorClosed	=	$0002;

	mcSDiscType		=	$0003;		{ "Disc type" selector. }
	mcS_CLV			=	$0001;
	mcS_CAV			=	$0002;
	mcS_CDV			=	$0003;
	mcS_CD			=	$0004;

	mcSDiscSize		=	$0004;		{ "Disc size" selector. }
	mcSDisc3inch	=	$0003;
	mcSDisc5inch	=	$0005;
	mcSDisc8inch	=	$0008;
	mcSDisc12inch	=	$000C;

	mcSDiscSide		=	$0005;		{ "Disc side" selector. }
	mcSSideOne		=	$0001;
	mcSSideTwo		=	$0002;


{ Time parameter values for MCGetTimes }

	mcElapsedTrack	=	$0000;		{ Elapsed time on current track/chapter. }
	mcRemainTrack	=	$0001;		{ Remaining time on current track/chapter. }
	mcElapsedDisc	=	$0002;		{ Elapsed time on disc. }
	mcRemainDisc	=	$0003;		{ Remaining time on disc. }
	mcTotalDisc		=	$0004;		{ Total run time on disc. }
	mcTotalFrames	=	$0005;		{ Total number of frames on disc. }
	mcTracks		=	$0006;		{ Binary start and ending track numbers 
									  (bits 31-16 = ending, 
									   bits 15-0 = starting track number.) }


{ Audio values }

	AudioOff		=	$0000;		{ Audio off. }
	AudioRight		=	$0001;		{ Audio thru right channel only. }
	AudioLinR		=	$0002;		{ Audio left in right only. }
	AudioMinR		=	$0003;		{ Audio mixed in right only. }
	AudioRinL		=	$0004;		{ Audio right in left only. }
	AudioRinLR		=	$0005;		{ Audio right in left and right. }
	AudioReverse	=	$0006;		{ Audio right in left, left in right. }
	AudioRinLMR		=	$0007;		{ Audio right in left, mixed in right. }
	AudioLeft		=	$0008;		{ Audio left channel only. }
	AudioStereo		=	$0009;		{ Audio both channels (in stereo). }
	AudioLinLR		=	$000A;		{ Audio left in left and right. }
	AudioLinLMR		=	$000B;		{ Audio left in left, mixed in right. }
	AudioMinL		=	$000C;		{ Audio mixed in left only. }
	AudioMinLRinR	=	$000D;		{ Audio mixed in left, right in right. }
	AudioMinLLinR	=	$000E;		{ Audio mixed in left, left in right. }
	AudioMonaural	=	$000F;		{ Audio mixed in left and right (monaural). }


{ Error codes }

	UnImp			=	$2601;		{ Unimplemented for this device. }
	NotApplic		=	$2601;		{ <see UnImp> }
	BadSpeed		=	$2602;		{ Invalid speed specified. }
	BadUnitType		=	$2603;		{ Invalid unit type specified. }
	TimeOutErr		=	$2604;		{ Timed out during device read. }
	NotLoaded		=	$2605;		{ No driver is currently loaded. }
	BadAudio		=	$2606;		{ Invalid audio value. }
	DevRtnError		=	$2607;		{ Device returned error (unable to perform). }
	UnRecStatus		=	$2608;		{ Unrecognizable status from device. }
	BadSelector		=	$2609;		{ Invalid selector value specified. }
	FunnyData		=	$260A;		{ Funny data received (try again). }
	InvalidPort		=	$260B;		{ Invalid port specified. }
	OnlyOnce		=	$260C;		{ Scans only once. }
	NoResMgr		=	$260D;		{ Resource manager not active. }
	ItemNotThere	=	$260E;		{ Item not found in database. }
	WasShutDown		=	$260F;		{ Media Control Toolset was already shut down. }
	WasStarted		=	$2610;		{ Media Control Toolset was already started. }
	BadChannel		=	$2611;		{ Invalid channel number. }
	InvalidParam	=	$2612;		{ Invalid parameter. }
	CallNotSupported =	$2613;		{ Call is not supported. }


{ Call definitions }

PROCEDURE MCShutDown; 							
INLINE $A2, $0326, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCStartUp(userID: INTEGER); 			
INLINE $A2, $0226, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCBinToTime(binTime: LongInt): LongInt; 				
INLINE $A2, $0D26, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCControl(channel, ctlCmd: INTEGER); 	
INLINE $A2, $1B26, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCGetDiscID(channel: INTEGER): LongInt;
INLINE $A2, $2826, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCGetErrorMsg(errNum: INTEGER; VAR errText: Str255); 						
INLINE $A2, $0926, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCGetName(channel: INTEGER; VAR str: Str255);
INLINE $A2, $2D26, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCGetNoTracks(channel: INTEGER): INTEGER;
INLINE $A2, $2926, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCGetPosition(channel, unitType: INTEGER): LongInt;
INLINE $A2, $2426, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCGetSpeeds(channel: INTEGER; VAR str: Str255); 							
INLINE $A2, $1D26, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCGetStatus(channel, statusSel: INTEGER): INTEGER; 				
INLINE $A2, $1A26, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCGetTimes(channel, timesSel: INTEGER): LongInt;
INLINE $A2, $2626, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCJog(channel, unitType: INTEGER; nJog: LongInt; jogRepeat: INTEGER);			
INLINE $A2, $2026, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCPause(channel: INTEGER); 			
INLINE $A2, $1826, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCPlay(channel: INTEGER); 			
INLINE $A2, $1726, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCRecord(channel: INTEGER); 			
INLINE $A2, $2A26, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCScan(channel, direction: INTEGER); 	
INLINE $A2, $1C26, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCSearchTo(channel, unitType: INTEGER; searchLoc: LongInt);
INLINE $A2, $2126, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCSearchWait(channel: INTEGER); 		
INLINE $A2, $2326, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCSetAudio(channel, ctlCommand: INTEGER); 							
INLINE $A2, $2526, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCSpeed(channel, framesPerSec: INTEGER); 								
INLINE $A2, $1E26, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCStop(channel: INTEGER); 			
INLINE $A2, $2B26, $22, $E10000, $8F, '_toolErr';

PROCEDURE MCStopAt(channel, unitType: INTEGER; stopLoc: LongInt);
INLINE $A2, $1F26, $22, $E10000, $8F, '_toolErr';

FUNCTION  MCTimeToBin: LongInt; 				
INLINE $A2, $0C26, $22, $E10000, $8F, '_toolErr';
