{---------------------------------------------------------------------

  file MediaControl.p
  
  Part of HyperCard IIGS 1.1

  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
	MediaControl channelNum, command
	
  This XCMD provides an interface to the Media Control Toolset
  (Tool 38) from HyperTalk.  It also enables or disables the
  video overlay feature of the Apple II Video Overlay card.
  
  Author:	Darin Acquistapace
  Created:	6/25/91
  Modified:	See Mod History Below


  Modification History:

	1/28/92 - Release today as sample code.
	
---------------------------------------------------------------------}

UNIT DummyUnit;

{$Z+ }
{$N+ }

INTERFACE

USES 	
	Types, Memory, MiscTool, Locator, GSOS, QuickDraw, Resources, QDAux, 
	Events, Controls, Windows, Menus, HyperXCMD, Video;
     
PROCEDURE EntryPoint(paramPtr: XCmdPtr);

IMPLEMENTATION

PROCEDURE MediaControl(paramPtr: XCmdPtr);											FORWARD;

PROCEDURE EntryPoint(paramPtr: XCmdPtr);
BEGIN
  MediaControl(paramPtr);
END;

{ These external variables are in MediaControl.a }
VAR
	{$J+}
	BTNRECT:			ARRAY [1..9] OF Rect;
	CDPIC:			Picture;
	PICRECSIZE:		LongInt;
	{$J-}

{ This routine is in MediaControl.a }
FUNCTION TranslateCommand(tableNum: INTEGER; str: Str255): INTEGER;			EXTERNAL;

{$I MediaControl.inc}

PROCEDURE MediaControl(paramPtr: XCmdPtr);

LABEL
   1;
	
CONST
	ScriptErrStr	= 	'Can''t understand arguments of XCMD MediaControl.';
	LoadErrStr		=	'Unable to load Media Control Toolset';
	StartErrStr		=	'Unable to start Media Control Toolset';
	InvalChanStr	=	'Invalid channel specified';
	UnknownCmdStr	=	'Unknown command';
	ParamRangeStr	=	'Parameter out of range';
	WindowErrStr	=	'Unable to create CD Audio window';
	WrongVersionStr= 	'MediaControl XCMD requires HyperCard IIGS 1.1';
	UnknownStatStr	=	'Unknown GetStatus argument';
	UnknownTimeStr	=	'Unknown GetTimes argument';
	UnknownTypeStr	=	'Unknown type of unit';
	UnknownControl	=	'Unknown control parameter';
	UnknownVidCmd	=	'Unknown video overlay command';
	BadTimeStr		=	'Invalid time format';
	WindName			=	'Media Control';
	DisplayXWind	=	'remote';
	OverlayStr		=	'videoOverlay';
	GlobalChannel	=	'mediaChannel';
	ResultVar		=	'mediaResult';
	VidStatString	=	'status';
	OnString			=	'on';
	OffString		=	'off';
	CopyrightStr	=	'MediaControl XCMD v1.0.1" & return & "by Darin Acquistapace, 6/25/91" & return & " 1991 Apple Computer, Inc.';
	HelpStr			=	'FORM:  MediaControl';
	MediaToolSet	=	38;
	VideoToolSet	=	33;
	MinChannel		= 	0;
	MaxChannel		=	8;

	{ Constants used when returning device status }
	StatString1		=	'laserdisc player';
	StatString2		=	'CD audio player';
	StatString3		=	'laserdisc/CD audio player';
	StatString4		=	'VCR';
	StatString5		=	'camcorder';
	StatString7		=	'playing';
	StatString8		=	'still';
	StatString9		=	'parked';
	StatString10	=	'open';
	StatString11	=	'closed';
	StatString12	=	'CLV';
	StatString13	=	'CAV';
	StatString14	=	'CDV';
	StatString15	=	'CD';
	StatString16	=	'3 inch';
	StatString17	=	'5 inch';
	StatString18	=	'8 inch';
	StatString19	=	'12 inch';
	StatString20	=	'side one';
	StatString21	=	'side two';
	NonDetermStr	=	'undetermined';
	
VAR
	numTracks:		INTEGER;		{ Remember to keep local variables order smallest }
	numParams:		INTEGER;		{ to largest so that most get into direct page }
	channelNum:		INTEGER;
	mediaCommand:	INTEGER;
	direction:		INTEGER;
	statType:		INTEGER;
	statArg:			INTEGER;
	timeArg:			INTEGER;
	statReturned:	INTEGER;
	timesType:		INTEGER;
	unitType:		INTEGER;
	controlNum:		INTEGER;
	startToolSet:	BOOLEAN;
	msgFromWind:	BOOLEAN;
	theDIscID:		LongInt;
	numUnits:		LongInt;
	stopLoc:			LongInt;
	searchLoc:		LongInt;
	discID:			LongInt;
	timeReturned:	LongInt;
	curPosition:	LongInt;
	cdXWindow:		WindowPtr;
	str: 				Str255;
	
	PROCEDURE ShutdownMediaControl;
	{ Shuts down the Media Control Toolset ONLY if we started it }
	BEGIN
	  IF startToolset THEN BEGIN
		  MCShutDown;
		  UnloadOneTool(MediaToolSet);
	  END; {if}
	END; {ShutdownMediaControl}
	
	
	PROCEDURE ErrResult(msg : Str255);
	{ Puts the specified string into the result and terminates }
	BEGIN
	  	ShutdownMediaControl;
		{ If the command came from the XWindow, display a dialog, otherwise
		  load the result }
		IF msgFromWind THEN
		  SendHCmessage(Concat('Answer "', msg, '" with "Cancel"'))
		ELSE
		  paramPtr^.returnValue := PasToZero(msg);
		EXIT(MediaControl);
	END; {ErrResult}
	
	
	PROCEDURE HTError;
	{ Puts the specified string into the result and terminates with a HyperTalk error}
	BEGIN
		ShutdownMediaControl;
		paramPtr^.returnValue := PasToZero(ScriptErrStr);
		paramPtr^.returnStat := 1;
		EXIT(MediaControl);
	END; {HTError}
	
	
	PROCEDURE CommandInfo(msg : str255);
	{ Puts the specified string into a dialog box and terminates }
	BEGIN
		SendHCmessage(Concat('Answer "', msg, '"')); 
		EXIT(MediaControl);
	END; {CommandInfo}
	
	
	PROCEDURE MyDisposeHandle(h: Handle);
	{ Disposes of a handle only if it is not NIL }
	BEGIN
	  IF h <> NIL THEN DisposeHandle(h);
	END; {MyDisposeHandle}
	
	
	PROCEDURE ReturnInVar(returnString: Str255);
	{ This sets the global "MediaResult" to the specified value }
	VAR
	  tempHandle:	Handle;
	BEGIN
	  tempHandle := PasToZero(returnString);
	  SetGlobal(ResultVar, tempHandle);
	  MyDisposeHandle(tempHandle);
	END; {ReturnInVar}

	
	PROCEDURE SetMediaChannel(channelNum: INTEGER);
   { Sets the global variable "MediaChannel" to the value specified }
	VAR
	  tempHandle:	Handle;
	BEGIN
	  tempHandle := PasToZero(LongToStr(channelNum));
	  SetGlobal(GlobalChannel, tempHandle);
	  MyDisposeHandle(tempHandle);
	END; {SetMediaChannel}
	

	FUNCTION GetMediaChannel: INTEGER;
	{ This returns the value of the global variable "MediaChannel."  If the
	  global is unitialized, it sets it to "1" and returns "1" }
	VAR
	  tempInt:		INTEGER;
	  tempHandle: 	Handle;
	  tempStr:	  	Str255;
	BEGIN
	  tempHandle := GetGlobal(GlobalChannel);
	  ZeroToPas(tempHandle^, tempStr);
	  MyDisposeHandle(tempHandle);
	  IF tempStr = '' THEN BEGIN
	    SetMediaChannel(1);
		 GetMediaChannel := 1;
		 EXIT(GetMediaChannel);
	  END; {if}
	  tempInt := StrToNum(tempStr);
	  IF tempInt < 1 THEN 
	    tempInt := 1
	  ELSE
	    IF tempInt > 8 THEN
		   tempInt := 8;
	  GetMediaChannel := tempInt;
	END; {GetMediaChannel}
	
	
	FUNCTION InBounds(sourceNum, minVal, maxVal: INTEGER): BOOLEAN;
	{ Returns true if sourceNum lies within minVal, maxVal inclusive,
	  false if it doesn't }
	BEGIN
	  InBounds := FALSE;
	  IF (sourceNum >= minVal) AND (sourceNum <= maxVal) THEN InBounds := TRUE;
	END; {InBounds}
	
	
	FUNCTION GetLongParam(paramNum: INTEGER): LongInt;
	{ This gets the specified parameter from the parameter block and converts
	  it to a LongInt.  If there are not exactly paramNum number of parameters,
	  we call HTError and exit. }
	BEGIN
	  IF numParams <> paramNum THEN HTError;
	  ZeroToPas(paramPtr^.params[paramNum]^, str);
	  GetLongParam := StrToLong(str);
	END; {GetLongParam}
	

	PROCEDURE CreateXWindow;
	{ Creates the CD Audio XWindow }
	CONST
	  WindLeft		= 50;
	  WindTop		= 50;
	  WindWidth		= 380;
	  WindHeight	= 26;
	VAR
	  windRect: 	Rect;
	BEGIN
	  SetRect(windRect, WindLeft, WindTop, WindLeft+WindWidth, WindTop+WindHeight);
	  { Create window visible }
	  cdXWindow := NewXWindow(windRect, windName, TRUE, xWindoidStyle); 
  	  IF cdXWindow = NIL THEN ErrResult(WindowErrStr);
	END; {CreateXWindow}
	

	PROCEDURE UpdateXWindow;
	{ This draws our background picture into the XWindow }
	VAR
	  tempPic:			Handle;
	  destRect:			Rect;
	  
	BEGIN
	  { Here we do something I really hate.  We have the picture hanging around
	    with our code but DrawPicture wants a handle to the picture.  We can't
		 send it a fake handle to our picture because it calls GetHandleSize on
		 the handle.  So...  we have to make a 2.5K handle, copy our picture to
		 it, and draw that, then dispose of the temp handle.  ...gross... }
		 
	  tempPic := NewHandle(PicRecSize, paramPtr^.userID, attrLocked+attrFixed, NIL);
	  IF tempPic = NIL THEN EXIT(UpdateXWindow);

	  BlockMove(@CDPic, tempPic^, PicRecSize);
	  
	  { Draw picture in upper left corner }
	  WITH CDPic.picFrame DO SetRect(destRect, 0, 0, right-left, bottom-top);
	  DrawPicture(PicHndl(tempPic), destRect);
	  
	  MyDisposeHandle(tempPic);
	END; {UpdateXWindow}


	FUNCTION CorrectVersion: BOOLEAN;
	{ This returns true if the version of HyperCard is >= minVersion }
	CONST
	  minVersion = '1.1';
	VAR
	  tempHandle:	Handle;
	  tempStr:		Str255;
	BEGIN
	  tempHandle := EvalExpr('the version');
	  ZeroToPas(tempHandle^, tempStr);
	  MyDisposeHandle(tempHandle);
	  CorrectVersion := tempStr >= minVersion;
	END; {CorrectVersion}


	FUNCTION HandleWindowClick(whichWindow: WindowPtr; ourEvent: EventRecord): INTEGER;
	{ Handle tracking a mouseClick in our window - return number of "button" clicked }
	CONST
	  numButtons	= 9;	{ How many buttons in our table }
	VAR
	  index:			INTEGER;
	  hitBtn:		INTEGER;
	  mouseLoc: 	Point;
	  oldInWind:	BOOLEAN;
	  inWind:		BOOLEAN;
	  targetRect:	Rect;
	BEGIN
	  HandleWindowClick := 0;
	  mouseLoc := ourEvent.where;
	  GlobalToLocal(mouseLoc);
		
	  hitBtn := 0;
	  { First find out which button was hit }
	  FOR index := 1 TO numButtons DO
		 IF PtInRect(mouseLoc, btnRect[index]) THEN BEGIN
		   hitBtn := index;
			LEAVE;
		 END; {if}
		  
	  IF hitBtn = 0 THEN EXIT(HandleWindowClick);   { No button was hit }
		
	  targetRect := btnRect[hitBtn];	{ Local for speed }
	  oldInWind := FALSE;
	  inWind := TRUE;
	  REPEAT
		 IF oldInWind <> inWind THEN BEGIN
		   InvertRect(targetRect);
		   oldInWind := inWind;
	    END; {if}
		 GetMouse(mouseLoc);
		 inWind := PtInRect(mouseLoc, targetRect);
	  UNTIL NOT Button(0);
	  IF oldInWind THEN InvertRect(targetRect);	{ Turn off hilite }
		
	  { Return # of "button" if mouseUp was in it }
	  IF inWind THEN HandleWindowClick := hitBtn;  
	END; {HandleWindowClick}


	FUNCTION HandleEvents: INTEGER;
	{ Handle events specific to our XWindow.  If the user clicked on of our
	  buttons, we return it so the code in the main proc can handle the
	  button press. }
	VAR
	  myEventInfo:	XWEventInfoPtr;
	  window:		WindowPtr;
	  event:			EventRecord;
	BEGIN
     myEventInfo := XWEventInfoPtr(paramPtr^.params[1]);
     window := myEventInfo^.eventWindow;
     event := myEventInfo^.event;
	  
	  HandleEvents := 0;
	  CASE event.what OF
	    xOpenEvt:			XWAllowReEntrancy(window, TRUE, TRUE);
	    updateEvt:			UpdateXWindow;
		 mouseDownEvt:		HandleEvents := HandleWindowClick(window, event);
		 xCursorWithin:	paramPtr^.passFlag := TRUE;		{ Let HC handle the cursor }
	  END; {case}
	END; {HandleEvents}


  FUNCTION GetLocFromUser: LongInt;
  { Display a dialog asking the user what track number to go to.
    Default value is the current track. }
  CONST
    askDialogStr = 'ask "Enter track to jump to:" with "';
  VAR
    curChap:		LongInt;
    tempHandle:	Handle;
	 chapStr:		Str31;
	 tempStr:		Str255;
  BEGIN
    chapStr := '';
	 curChap := 0;
    curChap := MCGetPosition(1, mcInChapters);
	 IF curChap > 0 THEN chapStr := NumToStr(curChap);
    GetLocFromUser := 0;
    SendHCMessage(Concat(askDialogStr, chapStr, '"'));
	 tempHandle := EvalExpr('it');
	 ZeroToPas(tempHandle^, tempStr);
	 MyDisposeHandle(tempHandle);
	 IF tempStr <> '' THEN 
	    GetLocFromUser := StrToLong(tempStr)
	  ELSE
	    GetLocFromUser := -1;
  END; {GetLocFromUser}
  
  
  FUNCTION StartUpVideoTool: BOOLEAN;
  { Loads and starts up the video overlay driver.  Returns true if we started
    it and need to shut it down, false o.w. }
  CONST
	 LoadErrStr			=	'Unable to load Video Toolset';
	 StartErrStr		=	'Unable to start Video Toolset';
  BEGIN
	 { Make sure Video Toolset is loaded}
	 LoadOneTool(VideoToolSet, 0);		{ Load Video Toolset, any version }
	 IF _toolErr <> noError THEN ErrResult(LoadErrStr);
	
	 VDStartUp;
	 IF (_toolErr = vdAlreadyStarted) OR (_toolErr = noError) THEN
	   StartUpVideoTool := NOT (_toolErr = vdAlreadyStarted)	{ Whether to shut it down }
	 ELSE BEGIN
		UnloadOneTool(VideoToolSet);
		ErrResult(StartErrStr);
	 END; {else}
  END; {StartUpVideoTool}
  
  
  PROCEDURE ShutdownVideoTool;
  { Shuts down and unloads the video overlay driver }
  BEGIN
    VDShutDown;
    UnloadOneTool(VideoToolSet);
  END; {ShutdownVideoTool}
  
  
  PROCEDURE DoOverlay(videoStatus: INTEGER);
  { Enables or disables the video overlay capabilities of the Apple II
    Video Overlay Card. }
  VAR
	 startVidTool:	BOOLEAN;
  BEGIN
    startVidTool := StartUpVideoTool;
	 
	 VDInControl(vdGenlock, videoStatus);		{ Enable/disable video overlay }

	 { Shut video toolset down ONLY if we started it }
	 IF startVidTool THEN ShutdownVideoTool;
  END; {DoOverlay}
  
  
  PROCEDURE ReturnOverlayStatus;
  { Returns the status (on or off) of the genlock feature of the Apple II 
    Video Overlay Card in the MediaResult global HT variable }
  VAR
	 startVidTool:	BOOLEAN;
  BEGIN
    startVidTool := StartUpVideoTool;
	 
	 IF VDInStatus(vdGenlock) = vdEnable THEN
	   ReturnInVar(OnString)
    ELSE
	   ReturnInVar(OffString);

	 { Shut video toolset down ONLY if we started it }
	 IF startVidTool THEN ShutdownVideoTool;
  END; {ReturnOverlayStatus}
  
  
  FUNCTION TimeToText(timeInBCD: LongInt): Str31;
  { Given a BCD time longint, this function converts it to a string in the
    format:  "hh hours,mm minutes,ss seconds" }
  VAR
    hours:		INTEGER;
	 minutes:	INTEGER;
	 seconds:	INTEGER;
    workTime:	Str31;
    hexVal:		Str31;
  BEGIN
    hexVal := NumToHex(timeInBCD, 8);
	 
	 workTime := '';
	 IF hexVal[1] <> '0' THEN workTime := hexVal[1];
	 workTime := Concat(workTime, hexVal[2], ' hour');
	 IF (hexVal[1] <> '0') OR (hexVal[2] <> '1') THEN workTime := Concat(workTime, 's');
	 workTime := Concat(workTime, ',');
  
	 IF hexVal[3] <> '0' THEN workTime := Concat(workTime, hexVal[3]);
	 workTime := Concat(workTime, hexVal[4], ' minute');
	 IF (hexVal[3] <> '0') OR (hexVal[4] <> '1') THEN workTime := Concat(workTime, 's');
	 workTime := Concat(workTime, ',');
  
	 IF hexVal[5] <> '0' THEN workTime := Concat(workTime, hexVal[5]);
	 workTime := Concat(workTime, hexVal[6], ' second');
	 IF (hexVal[5] <> '0') OR (hexVal[6] <> '1') THEN workTime := Concat(workTime, 's');
	 
	 TimeToText := workTime;
  END; {TimeToText}
  

  FUNCTION TextToTime(workStr: Str255): LongInt;
  { The compliment to TimeToText, this routine converts text in the form:
    "hh hours,mm minutes,ss seconds" to a BCD time LongInt. }
  VAR
	 value:		INTEGER;
	 index:		INTEGER;
	 textLen:	INTEGER;
	 totalTime:	LongInt;
    hours:		LongInt;
	 minutes:	LongInt;
	 seconds:	LongInt;
    oneToken:	Str255;
	 
  PROCEDURE SkipSpaces;
  { Moves index past any spaces }
  BEGIN
    WHILE (workStr[index] = ' ') AND (index < textLen) DO index := index + 1;
  END; {SkipSpaces}
  
  FUNCTION GetToken: Str255;
  { Returns all characters up to but not including a space or comma starting at
    index.  Moves index forward. }
  VAR
    tokenStr:	Str255;
  BEGIN
    tokenStr := '';
    WHILE (workStr[index] <> ' ') AND (index <= textLen) AND 
	 		 (workStr[index] <> ',') DO BEGIN
		tokenStr := Concat(tokenStr, workStr[index]);
	   index := index + 1;
    END; {while}
	 GetToken := tokenStr;
	 IF index < textLen THEN index := index + 1;
  END; {GetToken}
  
  BEGIN
    TextToTime := 0;
	 hours := 0;
	 minutes := 0;
	 seconds := 0;
    textLen := Length(workStr);
	 IF textLen = 0 THEN EXIT(TextToTime);
	 index := 1;
    REPEAT
	   SkipSpaces;
	   oneToken := GetToken;
		value := StrToNum(oneToken);
		SkipSpaces;
		CASE TranslateCommand(4, GetToken) OF
		  1,2:	hours := value;
		  3,4:	minutes := value;
		  5,6:	seconds := value;
		  OTHERWISE ErrResult(BadTimeStr);
  	   END; {case}
		SkipSpaces;
	 UNTIL index >= textLen;
	 totalTime := hours * 1000000 + minutes * 10000 + seconds * 100;
	 TextToTime := MCBinToTime(totalTime);
  END; {TextToTime}
  
BEGIN
 	numParams := paramPtr^.paramCount;
   msgFromWind := FALSE;			{ Command didn't come from the XWindow }
	startToolset := FALSE;			{ We haven't started anything yet }
	
	ReturnInVar('');					{ Clear media result }
	
	{ If the paramCount is negative, we have been called in response to an event }
	IF numParams < 0 THEN BEGIN
	  mediaCommand := HandleEvents;
	  msgFromWind := mediaCommand <> 0;
	  IF msgFromWind THEN GOTO 1;		{ Handle msg from XWindow }
	  EXIT(MediaControl);
	END; {if}

	IF numParams > 0 then BEGIN			
		ZeroToPas(paramPtr^.params[1]^, str);
		IF str = '!' THEN CommandInfo(CopyrightStr);		{ Display copyright notice }
		IF str = '?' THEN	CommandInfo(HelpStr);			{ Display correct usage }
	END ELSE
	  HTError;														{ We need at least one param }
		
   IF NOT CorrectVersion THEN ErrResult(WrongVersionStr);
	
	{ Create the CD Audio XWindow if the user specifies "Remote" }
	IF StringEqual(str, DisplayXWind) THEN BEGIN
	  CreateXWindow;
	  EXIT(MediaControl);
	END; {if}
	
	{ Handle the the "VideoOverlay" command.  Checked here since we don't
	  need to start up the Media Control Toolset to use the Video Overlay. }
	IF StringEqual(str, OverlayStr) THEN BEGIN
	  IF numParams <> 2 THEN HTError;
	  ZeroToPas(paramPtr^.params[2]^, str);
	  CASE TranslateCommand(6, str) OF
	    1:  DoOverlay(vdEnable);
		 2:  DoOverlay(vdDisable);
		 3:  ReturnOverlayStatus;
		 OTHERWISE ErrResult(UnknownVidCmd);
	  END; {case}
	  EXIT(MediaControl);
	END; {if}
	
	1:
	{ Make sure Media Control Toolset is loaded}
	LoadOneTool(MediaToolSet, 0);		{ Load Media Control Toolset, any version }
	IF _toolErr <> noError THEN ErrResult(LoadErrStr);
		
	MCStartUp(paramPtr^.userID);
	IF (_toolErr = WasStarted) OR (_toolErr = noError) THEN
	  startToolset := NOT (_toolErr = WasStarted)	{ Whether to shut it down }
	ELSE BEGIN
		UnloadOneTool(MediaToolSet);
		ErrResult(StartErrStr);
	END; {else}
	
	{ Get the command target channel number }
	IF msgFromWind THEN
	  channelNum := GetMediaChannel		{ Get channel from global }
	ELSE BEGIN
	  channelNum := StrToNum(str);
	  IF NOT InBounds(channelNum, MinChannel, MaxChannel) THEN
	    ErrResult(InvalChanStr);
	  
	  { Get the command }
	  ZeroToPas(paramPtr^.params[2]^, str);
	  
	  { Convert text specified by user into an index into the available commands }
	  mediaCommand := TranslateCommand(0, str);
	  IF mediaCommand = -1 THEN ErrResult(UnknownCmdStr);	
	END; {else}
	
	CASE mediaCommand OF
	  1: 		BEGIN
	     		  IF msgFromWind THEN
				    MCControl(channelNum, mcCEject)
				  ELSE BEGIN
	  			    ZeroToPas(paramPtr^.params[3]^, str);
				    controlNum := TranslateCommand(5, str);
	  			    IF controlNum = -1 THEN ErrResult(UnknownControl);
	  			    IF controlNum > 38 THEN { Handle audio commands }
					   MCSetAudio(channelNum, controlNum - 39)
					 ELSE BEGIN
					   CASE controlNum OF
						  11..18:	controlNum := controlNum + 29;
						  19..34:	controlNum := controlNum + 31;
						  35..36:	controlNum := controlNum + 65;
						  37..38:	controlNum := controlNum + 163;
						END; {case}
				      MCControl(channelNum, controlNum);
					 END; {else}
	  			  END; {else}
	  			END; {1}
	  2:		MCPlay(channelNum);
	  3:		MCStop(channelNum);
	  4:		MCPause(channelNum);
	  5,6:  	BEGIN
	   		  IF msgFromWind THEN BEGIN
				    numUnits := 1;
					 { Move forward in frames if option is down }
					 IF BAND(XWEventInfoPtr(paramPtr^.params[1])^.event.modifiers, 
					 		   optionKey) <> 0 THEN
					   unitType := mcInFrames
					 ELSE
					   unitType := mcInChapters;
				  END ELSE BEGIN
	  		       numUnits := GetLongParam(4);
	  			    ZeroToPas(paramPtr^.params[3]^, str);
				    unitType := TranslateCommand(3, str);
	  			    IF unitType = -1 THEN ErrResult(UnknownTypeStr);
					 IF unitType = mcInTime THEN BEGIN
					   ZeroToPas(paramPtr^.params[4]^, str);
						numUnits := TextToTime(str);
					 END; {if}
				  END; {else}
			     MCJog(channelNum, unitType, numUnits, 11-BSL(mediaCommand,1));
	  			END; {5,6}
	  7:  	BEGIN
	  			  IF msgFromWind THEN BEGIN
				    searchLoc := GetLocFromUser;
					 unitType := mcInChapters;
				  END ELSE BEGIN
	  		  	    searchLoc := GetLongParam(4);
	  			    ZeroToPas(paramPtr^.params[3]^, str);
				    unitType := TranslateCommand(3, str);
	  			    IF unitType = -1 THEN ErrResult(UnknownTypeStr);
					 IF unitType = mcInTime THEN BEGIN
					   ZeroToPas(paramPtr^.params[4]^, str);
						searchLoc := TextToTime(str);
					 END; {if}
				  END; {else}
				  IF searchLoc >= 0 THEN BEGIN
			  	    MCSearchTo(channelNum, unitType, searchLoc);
			  	    IF _toolErr = noError THEN BEGIN
					   MCSearchWait(channelNum);
				      IF msgFromWind THEN MCPlay(channelNum);
					 END; {if}
				  END; {if}
	  			END; {7}
	  8,9:	MCScan(channelNum, 17-BSL(mediaCommand,1));
	  
	  { Commands from here down can't be called from the XWindow }
	  10:  	BEGIN				
	  		  	  stopLoc := GetLongParam(4);
	  			  ZeroToPas(paramPtr^.params[3]^, str);
				  unitType := TranslateCommand(3, str);
	  			  IF unitType = -1 THEN ErrResult(UnknownTypeStr);
				  IF unitType = mcInTime THEN BEGIN
					 ZeroToPas(paramPtr^.params[4]^, str);
				    stopLoc := TextToTime(str);
				  END; {if}
			  	  MCStopAt(channelNum, unitType, stopLoc);
	  			END; {10}
	  11:		MCRecord(channelNum);
	  12:		BEGIN
	  			  theDiscID := MCGetDiscID(channelNum);
	    		  IF _toolErr = noError THEN ReturnInVar(LongToStr(theDiscID));
				END; {12}
	  13:		BEGIN
	   		  MCGetName(channelNum, str);
	  			  IF _toolErr = noError THEN ReturnInVar(str);
				END; {13}
	  14:		BEGIN
	  			  numTracks := MCGetNoTracks(channelNum);
				  IF _toolErr = noError THEN ReturnInVar(NumToStr(numTracks));
				END; {14}
	  15:		MCSpeed(channelNum, GetLongParam(3));
	  16:		BEGIN
	   		  MCGetSpeeds(channelNum, str);
	  			  IF _toolErr = noError THEN ReturnInVar(str);
				END; {16}
	  17:		BEGIN
	  			  IF numParams <> 3 THEN HTError;
	  			  ZeroToPas(paramPtr^.params[3]^, str);
				  statType := TranslateCommand(1, str);
	  			  IF statType = -1 THEN ErrResult(UnknownStatStr);	
	  			  CASE statType OF
				    1:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSDeviceType);
				  			  CASE statReturned OF
				    			 mcSLaserDisc:		ReturnInVar(StatString1);
				    			 mcSCDAudio:		ReturnInVar(StatString2);
				    			 mcSLaserCD:		ReturnInVar(StatString3);
				    			 mcSVCR:				ReturnInVar(StatString4);
				    			 mcSCamCorder:		ReturnInVar(StatString5);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {1}
				    2:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSPlayStatus);
				  			  CASE statReturned OF
				    			 mcSPlaying:		ReturnInVar(StatString7);
				    			 mcSStill:			ReturnInVar(StatString8);
				    			 mcSParked:			ReturnInVar(StatString9);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {2}
				    3:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSDoorStatus);
				  			  CASE statReturned OF
				    			 mcSDoorOpen:		ReturnInVar(StatString10);
				    			 mcSDoorClosed:	ReturnInVar(StatString11);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {3}
				    4:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSDiscType);
				  			  CASE statReturned OF
				    			 mcS_CLV:			ReturnInVar(StatString12);
				    			 mcS_CAV:			ReturnInVar(StatString13);
				    			 mcS_CDV:			ReturnInVar(StatString14);
				    			 mcS_CD:				ReturnInVar(StatString15);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {4}
				    5:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSDiscSize);
				  			  CASE statReturned OF
				    			 mcSDisc3inch:		ReturnInVar(StatString16);
				    			 mcSDisc5inch:		ReturnInVar(StatString17);
				    			 mcSDisc8inch:		ReturnInVar(StatString18);
				    			 mcSDisc12inch:	ReturnInVar(StatString19);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {5}
				    6:	BEGIN
					 		  statReturned := MCGetStatus(channelNum, mcSDiscType);
				  			  CASE statReturned OF
				    			 mcSSideOne:		ReturnInVar(StatString20);
				    			 mcSSideTwo:		ReturnInVar(StatString21);
					 			 OTHERWISE			ReturnInVar(NonDetermStr);
							  END; {case}
							END; {6}
				  END; {case}
	  			END; {17}
		18:	BEGIN
	  			  IF numParams <> 3 THEN HTError;
	  			  ZeroToPas(paramPtr^.params[3]^, str);
				  timesType := TranslateCommand(2, str);
	  			  IF timesType = -1 THEN ErrResult(UnknownTimeStr);
				  CASE timesType OF
				    1:	timeArg := mcElapsedTrack;
					 2:	timeArg := mcRemainTrack;
					 3:	timeArg := mcElapsedDisc;
					 4:	timeArg := mcRemainDisc;
					 5:	timeArg := mcTotalDisc;
					 6:	timeArg := mcTotalFrames;
					 7:	timeArg := mcTracks;
				  END; {case}
				  timeReturned := 0;
				  timeReturned := MCGetTimes(channelNum, timeArg);
				  IF _toolErr = 0 THEN BEGIN
				    CASE timeArg OF
					   mcTotalFrames:	ReturnInVar(LongToStr(timeReturned));
						mcTracks:		ReturnInVar(Concat(NumToStr(LoWrd(timeReturned)),
															',', NumToStr(HiWrd(timeReturned))));
					   OTHERWISE ReturnInVar(TimeToText(timeReturned));
					 END; {case}
				  END; {if}
				END; {18}
		19:	BEGIN
	  			  IF numParams <> 3 THEN HTError;
	  			  ZeroToPas(paramPtr^.params[3]^, str);
				  unitType := TranslateCommand(3, str);
	  			  IF unitType = -1 THEN ErrResult(UnknownTypeStr);
				  curPosition := MCGetPosition(channelNum, unitType);
				  IF _toolErr = 0 THEN BEGIN
				    IF unitType = mcInTime THEN
					   ReturnInVar(TimeToText(curPosition))
				    ELSE
					   ReturnInVar(LongToStr(curPosition));
				  END; {if}
				END; {19}
				
	END; {case}
	{ Have Media Control Toolset translate errors into text for the user }
	IF _toolErr <> noError THEN BEGIN
	  MCGetErrorMsg(_toolErr, str);
	  ErrResult(str);
	END; {if}
	
	{ Shut Media Control Toolset down ONLY if we started it }
	ShutdownMediaControl;

END; {MediaControl}

END.	{ of the dummy unit	}
