{---------------------------------------------------------------------

  file MemState.p
  
  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
	MemState [Coords]
	
	INPUTS:  Coords		(Optional) Upper left corner where window
						will appear.

  This XCMD creates and maintains an XWindow.  The XWindow contains
  status about the current memory state of the IIGS.  This includes
  free memory, memory available after purging, and the maximum
  contiguous memory block.
  
  Clicking on the XWindow toggles between text and bargraph views.
  Clicking the XWindow with the command key down compacts memory
  and purges all purgeable memory.
  
  Files:
  ------
    MemState.p
	MemState.r
	MakeFile
	
	
  Author:	Darin Acquistapace
  Created:	10/30/91
  Modified:	See Mod History Below

  Modification History:
  ---------------------
    10/30/91 - New today.
	12/05/91 - Slight cleanup of source.
	
---------------------------------------------------------------------}

UNIT DummyUnit;

{$Z+ }
{$N+ }

INTERFACE

USES 	
	Types, Memory, MiscTool, GSOS, QuickDraw, Resources, QDAux, Events, 
	Controls, Windows, Menus, HyperXCMD;
     
PROCEDURE EntryPoint(paramPtr: XCmdPtr);

IMPLEMENTATION

PROCEDURE MemState(paramPtr: XCmdPtr);									FORWARD;

PROCEDURE EntryPoint(paramPtr: XCmdPtr);
BEGIN
  MemState(paramPtr);
END;

PROCEDURE MemState(paramPtr: XCmdPtr);
CONST
	ScriptErrStr	= 'Can''t understand arguments of XCMD MemState.';
	CopyrightStr	= 'MemState XCMD v1.0.1" & return & "by Darin Acquistapace, 10/30/91" & return & " 1991 Apple Computer, Inc.';
	HelpStr			= 'FORM:  MemState [coords]';
	WrongVersionStr = 'MemState XCMD requires HyperCard IIGS 1.1';
	CreateErrStr	= 'Unable to create window';
	WindName		= 'Memory State';
	MemWidth		= 250;
	MemHeight		= 41;
	DefaultCoords	= $320032;		{ 50, 50 }
	
VAR
	pCount:			INTEGER;
	memWindow:		WindowPtr;
	windRect:		Rect;
	str:			Str255;
	
	PROCEDURE HTError;
	{ Puts the specified string into the result and terminates with a HyperTalk error}
	BEGIN
	  paramPtr^.returnValue := PasToZero(ScriptErrStr);
	  paramPtr^.returnStat := 1;
	  EXIT(MemState);
	END; {HTError}
	
	
	PROCEDURE ReturnResult(str: Str255);
	{ Puts the specified string into the result and terminates }
	BEGIN
	  paramPtr^.returnValue := PasToZero(str);
	  EXIT(MemState);
	END; {ReturnResult}
	
	
	PROCEDURE CommandInfo(msg: Str255);
	{ Puts the specified string into a dialog box and terminates }
	BEGIN
	  SendHCmessage(concat('answer "', msg, '"')); 
	  EXIT(MemState);
	END; {CommandInfo}


	PROCEDURE UpdateMyWindow(whichWindow: WindowPtr; fullUpdate: BOOLEAN);
	{ This updates the XWindow.  FullUpdate parameter determines whether
	  we draw the labels as well as the information. }
	CONST
	  TextLeft		= 18;
	  TextTop		= 10;
	  LineHeight	= 9;
	  ValueLeft		= 174;
	  
	  BarHeight		= 8;
	  BarWidth		= MemWidth - 40;
	  
	VAR
	  usedSize:			INTEGER;
	  purgeSize:		INTEGER;
	  myRealFreeMem:	LongInt;
	  myPurgeableMem:	LongInt;
	  myFreeMem:		LongInt;
	  myTotalMem:		LongInt;
	  myMaxBlock:		LongInt;
	  destRect:			Rect;
	  usedRect:			Rect;
	  purgeRect:		Rect;
	  boxRect:			Rect;
	  
	BEGIN
	  { Compute the stats }
	  myRealFreeMem := RealFreeMem;
	  myFreeMem := FreeMem;
	  myPurgeableMem := myRealFreeMem - myFreeMem;
	  myTotalMem := TotalMem;

	  IF BOOLEAN(GetXWindowValue(whichWindow)) THEN BEGIN  { Text display }
	  	{ Only compute MaxBlock for text display mode }
	  	myMaxBlock := MaxBlock;
		
	    { Draw the text strings }
	    SetTextMode(modeCopy);
        IF fullUpdate THEN BEGIN		{ Draw labels if necessary }
		  MoveTo(TextLeft, TextTop);
		  DrawString('Total memory:');
		  MoveTo(TextLeft, TextTop+LineHeight);
		  DrawString('Free memory:');
		  MoveTo(TextLeft, TextTop+LineHeight*2);
		  DrawString('Purgeable memory:');
		  MoveTo(TextLeft, TextTop+LineHeight*3);
		  DrawString('Max block:');
	    END; {if}
	  
	    SetRect(destRect, ValueLeft, 2, ValueLeft+100, TextTop+LineHeight*3);
	    EraseRect(destRect);
	  
	    MoveTo(ValueLeft, TextTop);
	    DrawString(LongToStr(myTotalMem));
	    MoveTo(ValueLeft, TextTop+LineHeight);
	    DrawString(LongToStr(myFreeMem));
	    MoveTo(ValueLeft, TextTop+LineHeight*2);
	    DrawString(LongToStr(myPurgeableMem));
	    MoveTo(ValueLeft, TextTop+LineHeight*3);
	    DrawString(LongToStr(myMaxBlock));
	  END ELSE BEGIN  { Bargraph display }
	  
	    { Figure rects first so we can draw everything relatively quickly }
	    usedSize := BAND(Trunc(((myTotalMem - myFreeMem - myPurgeableMem) / myTotalMem) 
						 * BarWidth), $fffe);	{ Keep on even pixel }
	    purgeSize := BAND(Trunc((myPurgeableMem / myTotalMem) * BarWidth) + 1, $fffe);

		SetRect(usedRect, 20, MemHeight - 13, 20 + usedSize, MemHeight - 13 + BarHeight);
		SetRect(purgeRect, usedRect.right-2, MemHeight - 13, usedRect.right + purgeSize, 
				MemHeight - 13 + BarHeight);
		SetRect(destRect, purgeRect.right-2, MemHeight - 13, BarWidth + 20, 
				MemHeight - 13 + BarHeight);

	    IF fullUpdate THEN BEGIN		{ Draw labels if necessary }
		  SetRect(boxRect, 20, 5, 36, 12);
		  SpecialRect(boxRect, 0, $4444);
		  MoveTo(46, 12);
		  DrawString('Used');
		  SetRect(boxRect, 20, 16, 36, 23);
		  SpecialRect(boxRect, 0, $6666);
		  MoveTo(46, 23);
		  DrawString('Purgeable');
		  SetRect(boxRect, 140, 5, 156, 12);
		  SpecialRect(boxRect, 0, $eeee);
		  MoveTo(166, 12);
		  DrawString('Free');
		END; {if}
	    
		{ These are grouped together to draw all three boxes quickly }
		SpecialRect(usedRect, 0, $4444);			{ Used memory - red }
		SpecialRect(purgeRect, 0, $6666);			{ Purgeable - orange }
		SpecialRect(destRect, 0, $eeee);			{ Free memory - yellow }
	  
	  END; {else}
	END; {UpdateMyWindow}


    PROCEDURE HandleWindowClick(whichWindow: WindowPtr; modifiers: INTEGER);
	VAR
	  destRect:	Rect;
	BEGIN
	  IF BAND(modifiers, appleKey) = 0 THEN BEGIN	{ If command key isn't down }
	    { Toggle between text and bar graph views }
	    SetXWindowValue(whichWindow, LongInt(NOT BOOLEAN(GetXWindowValue(whichWindow))));
		SetRect(destRect, 0, 0, MemWidth, MemHeight);
		EraseRect(destRect);
		UpdateMyWindow(whichWindow, TRUE);	{ Redraw everything }
	  END ELSE BEGIN		{ Compact memory and redraw }
	    PurgeAll(0);
		CompactMem;
		UpdateMyWindow(whichWindow, FALSE);
	  END; {else}
	END; {HandleWindowClick}
	
	
	PROCEDURE HandleEvents;
	{ Handle events specific to our XWindow }
	CONST
	  IntervalTime 	= 5 * 60;		{ Every five seconds, 300 ticks }
	VAR
	  curMode:		BOOLEAN;
	  myEventInfo:	XWEventInfoPtr;
	  window:		WindowPtr;
	  event:		EventRecord;
	BEGIN
      myEventInfo := XWEventInfoPtr(paramPtr^.params[1]);
      window := myEventInfo^.eventWindow;
      event := myEventInfo^.event;
	  
	  CASE event.what OF
	    xOpenEvt:			BEGIN	{ Allow recursion, set idle time }
							  XWAllowReEntrancy(window, TRUE, TRUE);
							  SetXWIdleTime(window, IntervalTime);
							END; {xOpenEvt}
		nullEvt, updateEvt:	UpdateMyWindow(window, event.what = updateEvt);
		mouseDownEvt:		HandleWindowClick(window, event.modifiers);
		xCursorWithin:		paramPtr^.passFlag := TRUE; { Let HC change our cursor }
	  END; {case}
	END; {HandleEvents}

	
	FUNCTION CorrectVersion: BOOLEAN;
	{ This returns true if the version of HyperCard is >= minVersion }
	CONST
	  minVersion = '1.1';
	VAR
	  tempHandle:	Handle;
	  tempStr:		Str255;
	BEGIN
	  tempHandle := EvalExpr('the version');
	  ZeroToPas(tempHandle^, tempStr);
	  IF tempHandle <> NIL THEN DisposeHandle(tempHandle);
	  CorrectVersion := tempStr >= minVersion;
	END; {CorrectVersion}
	

BEGIN
    pCount := paramPtr^.paramCount;
  
    { If the paramCount is negative, we have been called in response to an event }
  	IF pCount < 0 THEN BEGIN
	  HandleEvents;
	  EXIT(MemState);
  	END; {if}

	{ Display help or copyright info in response to "?" or "!" }
	IF pCount = 1 THEN BEGIN
	  ZeroToPas(paramPtr^.params[1]^, str);
	  IF str = '!' THEN CommandInfo(CopyrightStr);
	  IF str = '?' THEN CommandInfo(HelpStr);
	END; {if}
	
	{ We'll take zero or one parameter }
	IF pCount > 1 THEN HTError;

	{ Make sure we are running at least version 1.1 of HyperCard IIGS }
    IF NOT CorrectVersion THEN ReturnResult(WrongVersionStr);
	
	{ Set coordinates for the window, use specified or default if none }
	WITH windRect DO BEGIN
	  IF pCount = 1 THEN BEGIN
	    StrToPoint(str, topLeft);
		left := left * 2;			{ Adjust to 640 coords }
	  END ELSE
	    topLeft := Point(DefaultCoords);
		
	  right := left + MemWidth;
	  bottom := top + MemHeight;
	END; {with}	
		
	{ Create window initially visible }
	memWindow := NewXWindow(windRect, WindName, TRUE, xWindoidStyle);
    IF memWindow = NIL THEN ReturnResult(CreateErrStr);

END;

END.	{ of the dummy unit	}