{---------------------------------------------------------------------

  file PBeep.p
  
  Sample XCMD for HyperCard IIGS

  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
	PBeep		beep once
	PBeep ##	beep n times
	PBeep ?		display usage information
	PBeep !		display version information
	
  This sample XCMD illustrates the implementation of an external
  command for use with HyperCard IIGS.
  
  Author:	Darin Acquistapace
  Created:	6/25/91
  Modified:	See Mod History Below


  Modification History:

	06/25/91 - New today.
	
---------------------------------------------------------------------}

UNIT DummyUnit;

{$Z+}		{ Not strictly needed, however these flags make the output	}
{$N+}		{ easier to read when an XCMD is disassembled with DumpObjIIGS	}

INTERFACE

USES 
	Types, MiscTool, QuickDraw, GSOS, Resources, Memory, HyperXCMD;
     
PROCEDURE EntryPoint(paramPtr: XCmdPtr);

IMPLEMENTATION

PROCEDURE PBeep(paramPtr: XCmdPtr);									FORWARD;

PROCEDURE EntryPoint(paramPtr: XCmdPtr);
BEGIN
  PBeep(paramPtr);
END;

PROCEDURE PBeep(paramPtr: XCmdPtr);
CONST
	MinBeeps		= 1;		{ Minimum number of beeps }
	MaxBeeps		= 15;		{ Maximum number of beeps }
	
	ScriptErrStr	= 'Can''t understand arguments of XCMD PBeep.';
	CopyrightStr	= 'PBeep XCMD v1.1" & return & "by Darin Acquistapace, 6/25/91" & return & " 1991 Apple Computer, Inc.';
	HelpStr			= 'FORM:  PBeep NumberOfBeeps';
	WrongVersionStr = 'PBeep XCMD requires HyperCard IIGS 1.0';

VAR
	numParams:		INTEGER;
	counter:		INTEGER;
	beepCount:		INTEGER;
	str:			Str255;
	
	PROCEDURE HTError;
	{ Puts the specified string into the result and terminates with a HyperTalk error}
	BEGIN
	  ParamPtr^.ReturnValue := PasToZero(ScriptErrStr);
	  ParamPtr^.ReturnStat := 1;
	  EXIT(PBeep);
	END; {HTError}
	
	
	PROCEDURE ReturnResult(str: Str255);
	{ Puts the specified string into the result and terminates }
	BEGIN
	  ParamPtr^.ReturnValue := PasToZero(str);
	  EXIT(PBeep);
	END; {ReturnResult}
	
	
	PROCEDURE CommandInfo(Msg : str255);
	{ Puts the specified string into a dialog box and terminates }
	BEGIN
	  SendHCmessage(Concat('answer "', Msg, '"')); 
	  EXIT(PBeep);
	END; {CommandInfo}
	
	
	FUNCTION PinWord(input, min, max: INTEGER): INTEGER;
	{ Pin an input value between two bounding integers }
	BEGIN
	  IF input < min THEN 
	  	PinWord := min
	  ELSE
	    IF input > max THEN 
	  	  PinWord := max
	    ELSE
	      PinWord := input;
	END; {PinWord}
		
	
	FUNCTION CorrectVersion: BOOLEAN;
	{ This returns true if the version of HyperCard is >= minVersion }
	CONST
	  minVersion = '1.0';
	VAR
	  tempHandle:	Handle;
	  tempStr:		Str255;
	BEGIN
	  tempHandle := EvalExpr('the version');
	  ZeroToPas(tempHandle^, tempStr);
	  IF temphandle <> NIL THEN DisposeHandle(tempHandle);
	  CorrectVersion := tempStr >= minVersion;
	END; {CorrectVersion}
	

BEGIN
	numParams := paramPtr^.paramCount;
	
	{ Display help or copyright info in response to "?" or "!" }
	IF numParams > 0 THEN BEGIN
	  ZeroToPas(paramPtr^.params[1]^, str);
	  IF str = '!' THEN CommandInfo(CopyrightStr);
	  IF str = '?' THEN CommandInfo(HelpStr);
	END; {if}
	
	IF numParams > 1 THEN HTError;	{ Display HyperTalk error unless 0 or 1 param }

	{ Make sure we are running at least version 1.0 of HyperCard IIGS }
    IF NOT CorrectVersion THEN ReturnResult(WrongVersionStr);
	
	{ Get how many times to beep, default to MinBeeps if not specified }
	IF numParams = 1 THEN
		beepCount := PinWord(StrToNum(str), MinBeeps, MaxBeeps)
	ELSE
		beepCount := MinBeeps;
		
  	FOR counter := 1 TO beepCount DO SysBeep;
END; {PBeep}

END.	{ of the dummy unit	}