{---------------------------------------------------------------------

  file Picture.p
  
  Part of HyperCard IIGS 1.1

  Copyright  1991 Apple Computer, Inc.

  This XCMD has the following syntax:
	
	Picture [SourceName [, SourceType [, windowStyle [, visible [, rect ]]]]]
	
	INPUTS: sourceName	Reference to pict to draw in the window
			sourceType	Type of pict, resource, clipboard, or file 
			windowStyle	Style of the window
			Visible		Initial visibility
			Rect		Rectangle defining the content region of the window
			
  This XCMD creates and maintains an XWindow.  The XWindow contains
  a picture retrieved from a resource, file, or the clipboard with a name 
  or id specified by the caller.
  
  Authors:	Eric Schlosser, Darin Acquistapace
  Created:	5/22/91
  ModIFied:	See Mod History Below


  ModIFication History:

	05/22/91 - New today.
	12/05/91 - Release as sample source.
   
---------------------------------------------------------------------}

UNIT DummyUnit;

{$Z+ }
{$N+ }

INTERFACE

USES 	
	Types, Memory, MiscTool, GSOS, QuickDraw, Resources, QDAux, Events, 
	Controls, Windows, Menus, IntMath, Scrap, HyperXCMD;
     
PROCEDURE EntryPoint(paramPtr: XCmdPtr);

IMPLEMENTATION

CONST
  ScrollWidth		= 28;			{ Width of vertical scrollbar }
  ScrollHeight		= 12;			{ Height of horizontal scrollbar }
  PageUpFlag		= 1;
  PageDnFlag		= 2;
  UpScrollAmt		= -10;			{ Amounts to scroll by arrow }
  DnScrollAmt		= 10;
  
TYPE
  wDataPtr = ^wDataRec;				{ Record for maintaining each window }
  wDataHndl = ^wDataPtr;
  wDataRec = RECORD
	vScroll:		CtlRecHndl;
	hScroll:		CtlRecHndl;
	growBox:		CtlRecHndl;
	myPictureHndl: 	PicHndl;
	windowType: 	INTEGER;
	usesScrollBars:	BOOLEAN;
  END; {record}

PROCEDURE XPicture(paramPtr: XCmdPtr);									FORWARD;

PROCEDURE EntryPoint(paramPtr: XCmdPtr);
BEGIN
  XPicture(paramPtr);
END;

{ External assembly procedure which unpacks MacPaint style pictures }
FUNCTION ImportMacPaint(refNum: INTEGER; dstBuf: Ptr):	BOOLEAN;		EXTERNAL;

{ These next procedures are not nested within the XPicture procedure so
  that they can be called by the toolbox }
PROCEDURE UpdatePicture(whichWindow: WindowPtr);
{ This handles drawing our picture into the XWindow in response to update
  events or scrolling }

VAR
  destRect:			Rect;
  rgnRect:			Rect;
  newVisRgn:		RgnHandle;
  windowTemp:		wDataHndl;		

BEGIN
  windowTemp := wDataHndl(GetXWindowValue(whichWindow));
  HLock(Handle(windowTemp));
  WITH windowTemp^^ DO BEGIN
	{ Leave if no picture exists }
	IF myPictureHndl = NIL THEN EXIT(UpdatePicture);
  
	{ Draw picture in upper left corner }
	WITH myPictureHndl^^.picFrame DO
	  SetRect(destRect, 0, 0, right - left, bottom - top);

	{ Handle offset of picture based on scrollbar values }
	IF usesScrollBars THEN BEGIN
	  WITH whichWindow^.portRect DO 
		SetRect(rgnRect, 0, 0, right - ScrollWidth, bottom - ScrollHeight);
  
	  { Protect scroll bars from being overwritten }
	  newVisRgn := NewRgn;
	  GetClip(newVisRgn);
	  ClipRect(rgnRect);
  
	  { Move output rect based on scroll }
	  OffsetRect(destRect, BAND(GetCtlValue(hScroll), $FFFE) * (-1),
				 GetCtlValue(vScroll) * (-1));
	END; {if}

	DrawPicture(myPictureHndl, destRect);			{ Plot the picture }
  
	IF usesScrollBars THEN BEGIN					{ Restore clip }
	  SetClip(NewVisRgn);
	  DisposeRgn(newVisRgn);
	END; {if}
  END; {with}
  HUnlock(Handle(windowTemp));
END; {UpdatePicture}


PROCEDURE MoveScroll(foundControl: CtlRecHndl; whichWindow: WindowPtr;
					 scrollAmount: INTEGER);
{ Adjust the scroll value of the specified scrollbar control }
VAR
  origValue:	INTEGER;
  cntrlValue: 	INTEGER;
  dataSize:		INTEGER;
  viewSize:		INTEGER;
  controlParam:	LongInt;
BEGIN
  controlParam := GetCtlParams(foundControl);
  dataSize := LoWord(controlParam);
  viewSize := HiWord(controlParam);

  origValue := GetCtlValue(foundControl);
  CASE scrollAmount OF
	PageUpFlag:	cntrlValue := origValue - viewSize;
	PageDnFlag:	cntrlValue := origValue + viewSize;
	OTHERWISE cntrlValue := origValue + scrollAmount;
  END; {case}
	
  { Keep from retreating past zero }
  IF cntrlValue < 0 THEN cntrlValue := 0;
  
  { Keep from bopping off the end }
  IF cntrlValue + viewSize > dataSize THEN cntrlValue := dataSize - viewSize;
	  
  { If scroll has changed, update the control and picture }
  IF cntrlValue <> origValue THEN BEGIN
	SetCtlValue(cntrlValue, foundControl);
	UpdatePicture(whichWindow);
  END; {if}
END; {MoveScroll}


PROCEDURE ScrollAction(partCode: INTEGER; hitCtrl: CtlRecHndl);
{ Move the scroll depending on the part of the scroll clicked }
BEGIN
  WITH hitCtrl^^ DO BEGIN
	CASE partCode OF
	  upArrow:		MoveScroll(hitCtrl, ctlOwner, UpScrollAmt);
	  pageUp:		MoveScroll(hitCtrl, ctlOwner, PageUpFlag);
	  downArrow:	MoveScroll(hitCtrl, ctlOwner, DnScrollAmt);
	  pageDown:		MoveScroll(hitCtrl, ctlOwner, PageDnFlag);
	END; {case}
  END; {with}
END; {ScrollAction}
	
	
PROCEDURE XPicture(paramPtr: XCmdPtr);
CONST
	ScriptErrStr	= 'Can''t understand arguments of XCMD Picture.';
	CopyrightStr	= 'Picture XCMD v1.0.1" & return & "by Eric Schlosser 9/13/91" & return & " 1991 Apple Computer, Inc.';
	HelpStr			= 'FORM: Picture [SourceName [, SourceType " & return & "[, windowStyle [Visible [,Rect ]]]]]';
	WrongVersionStr = 'Picture XCMD requires HyperCard IIGS 1.1';
	CreateErrStr	= 'Unable to create window';
	NoResourceStr	= 'Can''t find that resource';
	NoFileStr	    = 'Can''t find that picture file';
	NoClipStr	    = 'No picture on clipboard';
	WaitCursorStr	= 'set cursor to watch';
	RectStr1		= 'rect';
	RectStr2		= 'rectangle';
	ShadowStr		= 'shadow';
	DialogStr		= 'dialog';
	DocumentStr		= 'document';
	WindoidStr		= 'windoid';
	FileStr			= 'file';
	ResourceStr		= 'resource';
	ClipStr			= 'clipboard';
	
VAR
	picXWindow:		WindowPtr;
	wData:			wDataHndl;
	windRect:		Rect;
	myPicHandle:	PicHndl;
	nameHandle:		Handle;
	pCount:			INTEGER;
	windowStyle:	INTEGER;
	myID:	   		INTEGER;
	picWidth:		INTEGER;
	picHeight:		INTEGER;
	visible: 		BOOLEAN;
	hasScrollBars:	BOOLEAN;
	str:			Str255;
	tempStr:		Str255;
	 
	PROCEDURE MyDisposeHandle(h: Handle);
	{ Disposes of a handle only if it is not nil }
	BEGIN
	  IF h <> NIL THEN DisposeHandle(h);
	END; {MyDisposeHandle}
	
	PROCEDURE ReturnResult(str: Str255);
	{ Puts the specified string into the result and terminates }
	BEGIN
	  paramPtr^.returnValue := PasToZero(str);
	  EXIT(XPicture);
	END; {ReturnResult}
	
	PROCEDURE HTError;
	{ Puts the specified string into the result and terminates with a HyperTalk error }
	BEGIN
	  paramPtr^.returnStat := 1;
	  ReturnResult(ScriptErrStr);
	END; {HTError}
	
	PROCEDURE CommandInfo(msg: Str255);
	{ Puts the specified string into a dialog box and terminates }
	BEGIN
	  SendHCmessage(concat('answer "', msg, '"')); 
	  EXIT(XPicture);
	END; {CommandInfo}
	
	PROCEDURE HandleErrors(msg: Str255);
	BEGIN
	  SendHCMessage(concat('answer"', msg,'"'));
	  paramPtr^.returnValue := pastoZero(msg);
	  EXIT(XPicture);
	END;

	FUNCTION NewVScroll(whichWindow: WindowPtr; whichPicture: PicHndl;
					    vPos: INTEGER): CtlRecHndl;
	{ Returns a Handle to a new Vert. Scrollbar }
	VAR
	  vScrollBar:  ScrollBarTemplate;
	BEGIN
	  WITH vScrollBar, vScrollBar.ctlTemplate, whichWindow^.portRect DO BEGIN
		pCount := 9;
		ID := 711;
		SetRect(rect, right - ScrollWidth, -1, right + 2, bottom - 11);
		procRef := ScrollBarControl;
		flag := 03;
		moreFlags := $1000;
		refCon := 0;
		
		{ Maximum size is the size of the picture }
		WITH whichPicture^^.picFrame DO maxSize := bottom - top;
		  
	    { View size is the size of the window minus scroll bars }
		viewSize := bottom - top - scrollHeight;
		
  		{ Keep from bopping off the end }
  		IF vPos + viewSize > maxSize THEN vPos := maxSize - viewSize;

		{ If view is bigger than picture, keep scroll at zero }
		IF viewSize >= maxSize THEN vPos := 0;
		initalValue := vPos;
	  END; {with}
	
	  NewVScroll := NewControl2(whichWindow, singlePtr, ref(@VScrollBar));
	  IF _toolErr <> 0 THEN NewVScroll := NIL;	
	END; {NewVScroll}
		
	FUNCTION NewHScroll(whichWindow: WindowPtr; whichPicture: PicHndl;
					    hPos: INTEGER): CtlRecHndl;
	{ Returns a Handle to a new Horz. Scrollbar }
	VAR
	  hScrollBar:  ScrollBarTemplate;
	BEGIN
	  WITH hScrollBar, hScrollBar.ctlTemplate, whichWindow^.portRect DO BEGIN
		pCount := 9;
		ID := 712;
		SetRect(rect, -2, bottom-ScrollHeight, right-ScrollWidth+2, bottom+1);
		procRef := ScrollBarControl;
		flag := $1c;
		moreFlags := $1000;
		refCon := 0;
		
		{ Maximum size is the size of the picture }
		WITH whichPicture^^.picFrame DO maxSize := right - left;

	    { View size is the size of the window minus scroll bars }
		viewSize := right - left - scrollWidth;

  		{ Keep from bopping off the end }
  		IF hPos + viewSize > maxSize THEN hPos := maxSize - viewSize;

		{ If view is bigger than picture, keep scroll at zero }
		IF viewSize >= maxSize THEN hPos := 0;
		initalValue := hPos;
	  END; {WITH}
	
	  NewHScroll := NewControl2(whichWindow, singlePtr, ref(@hScrollBar));
	  IF _toolErr <> 0 THEN NewHScroll := NIL;	
	END; {NewHScroll}
		
	FUNCTION NewGrowbox(whichWindow: WindowPtr): CtlRecHndl;
	{ Returns a handle to a new growbox control }
	VAR
	  myGrowbox:  SizeBoxTemplate;
	BEGIN
	  WITH myGrowbox.ctlTemplate, whichWindow^.portRect DO BEGIN
		pCount := 6;
		ID := 713;
		SetRect(rect, right-ScrollWidth, bottom-ScrollHeight, right+2, bottom+1);
		procRef := growControl;
		flag := 01;
		moreFlags := $1000;
		refCon := 0;
	  END; {WITH}

	  NewGrowbox := NewControl2(whichWindow,singlePtr,ref(@MyGrowbox));
	  IF _toolErr <> 0 THEN NewGrowbox := NIL;	
	END; {NewGrowbox}


	FUNCTION GetWindowHt(whichpic: PicHndl; scroll: BOOLEAN): INTEGER;
	VAR
	  workHeight: INTEGER;
	BEGIN
	  WITH whichpic^^.picFrame DO BEGIN
		workHeight :=  bottom - top;
		IF scroll THEN workHeight := workHeight + scrollHeight;
	  END; {with}
	  GetWindowHt := workHeight;
	END; {GetWindowHt}


	FUNCTION GetWindowWth(whichpic: PicHndl; scroll: BOOLEAN): INTEGER;
	VAR
	  workWidth: INTEGER;
	BEGIN
	  WITH whichpic^^.picFrame DO BEGIN
		workWidth := right - left;
		IF scroll THEN workWidth := workWidth + scrollWidth;
	  END; {with}
	  GetWindowWth := workWidth;
	END; {GetWindowWth}


	PROCEDURE CleanUpMemory(whichWindow: WindowPtr);
	{ Remove our picture from memory when we close }
	BEGIN
	  IF wData^^.myPictureHndl <> NIL THEN KillPicture(wData^^.myPictureHndl);
	  MyDisposeHandle(Handle(wData));
	  SetXWindowValue(whichWindow, 0);   { Just to be safe - clear this value }
	END; {CleanUpMemory}
	
	
	PROCEDURE SizeBox(whichWindow: WindowPtr; foundControl: CtlRecHndl);
	{ Handle moving the size box and creating new scroll bars in the
	  correct positions }
	VAR
	  hPos: INTEGER;
	  vPos: INTEGER;
	BEGIN
	  WITH whichWindow^.portRect DO 
	    MoveControl(right-ScrollWidth, bottom-ScrollHeight, foundControl);
		
	  HLock(Handle(wData));
	  WITH wData^^ DO BEGIN
	    hPos := GetCtlValue(hScroll);
	    vPos := GetCtlValue(vScroll);
		
		{ Get rid of the old scroll bars }
	    DisposeControl(vScroll);
	    DisposeControl(hScroll);
		
		{ And create new ones in the correct positions }
	    vScroll:= NewVScroll(whichWindow, myPictureHndl, vPos);
	    hScroll:= NewHScroll(whichWindow, myPictureHndl, hPos);
	  END; {with}
	  HUnlock(Handle(wData));
	END;
	
	PROCEDURE CopyPicToClip(modifiers: INTEGER);
	{ Copies the current picture to the clipboard if appleKey was down }
	LABEL 9;
	VAR
	  saveError:  INTEGER;
	BEGIN
	  IF BAND(modifiers, appleKey) = 0 THEN EXIT(CopyPicToClip);
	  
	  SendHCMessage(WaitCursorStr);
	  ZeroScrap;
	  IF _toolErr <> noError THEN GOTO 9;
	  HLock(Handle(wData^^.MyPictureHndl));
	  PutScrap(GetHandleSize(Handle(wData^^.myPictureHndl)), picScrap, 
	  		   Ptr(wData^^.MyPictureHndl^));
	  saveError := _toolErr;
	  HUnlock(Handle(wData^^.MyPictureHndl));
	  IF saveError = noError THEN EXIT(CopyPicToClip);
	  
	  9:
	  Sysbeep;
	END; {CopyPicToClip}
	
	
	PROCEDURE HandleWindowClick(whichWindow: WindowPtr; ourEvent: EventRecord);
	{ The user has clicked in our window, handle tracking the click including
	  resizing, scrolling, and copying the picture to the clipboard }
	CONST
	  sizeBoxClick	= $80;
    VAR
	  myPart:		INTEGER;
	  procAddr:		Ptr;
	  foundControl: CtlRecHndl;
	BEGIN
	  WITH ourEvent.where DO BEGIN
	    myPart := FindControl(foundControl, h, v, whichWindow);
	    IF myPart > 0 THEN BEGIN
		  procAddr := NIL;
		  IF (myPart = upArrow) OR (myPart = downArrow) OR
		     (myPart = pageUp) OR (pageDown = pageDown) THEN
		       procAddr := @ScrollAction;
	      CASE TrackControl(h, v, procAddr, foundControl) OF
		    thumb:			UpdatePicture(whichWindow);
		    sizeBoxClick:  	SizeBox(whichWindow, foundControl);
	      END; {case}
	    END ELSE
		  CopyPicToClip(ourEvent.modifiers);
	  END; {with}
	END; {HandleWindowClick}
	

	PROCEDURE HandleEvents;
	{ Handle events specific to our XWindow }
	VAR
	  myEventInfo:	XWEventInfoPtr;
	  window:		WindowPtr;
	  event:		EventRecord;
	BEGIN
      myEventInfo := XWEventInfoPtr(paramPtr^.params[1]);
      window := myEventInfo^.eventWindow;
      event := myEventInfo^.event;
	  wData := wDataHndl(GetXWindowValue(window));
	  CASE event.what OF
	    xOpenEvt:			XWAllowReEntrancy(window, TRUE, TRUE);
	    updateEvt:			BEGIN
							  DrawControls(window);
							  UpdatePicture(window);
							END;
		mouseDownEvt:		HandleWindowClick(window, event);
  		xCloseEvt:			CleanUpMemory(window);
		xCursorWithin:		paramPtr^.passFlag := TRUE; { Let HC change our cursor }
	  END; {case}
	END; {HandleEvents}


	PROCEDURE FSClose(fileRef: INTEGER);
	{ Just a way to call CloseGS without needing a parameter block }
	VAR parmBlock:	RefNumRecGS;
	BEGIN
	  parmBlock.pCount := 1;
	  parmBlock.refNum := fileRef;
	  CloseGS(parmBlock);
	END;
	
	FUNCTION FSRead(fileRef: INTEGER; dataBuf: Ptr; reqCount: LongInt): LongInt;
	{ Just a way to call ReadGS without needing a parameter block }
	VAR parmBlock:	IORecGS;
	BEGIN
	  parmBlock.pCount := 5;
	  parmBlock.refNum := fileRef;
	  parmBlock.dataBuffer := dataBuf;
	  parmBlock.requestCount := reqCount;
	  parmBlock.cachePriority := cacheOff;
	  ReadGS(parmBlock);
	  FSRead := parmBlock.transferCount;
	END;
		
	PROCEDURE FSSetMark(fileRef: INTEGER; mark: LongInt);
	{ Set the mark of a file }
	VAR parmBlock:	SetPositionRecGS;
	BEGIN
	  parmBlock.pCount := 3;
	  parmBlock.refNum := fileRef;
	  parmBlock.base := 0;
	  parmBlock.displacement := mark;
	  SetMarkGS(parmBlock);
	END;
	
	FUNCTION GetPictFile(name: Str255): PicHndl;
	{ Imports a picture, APF or MacPaint.  If name is empty, we present 
	  a std getfile dialog.  Otherwise, we try to open the named file }
	LABEL 8,9;
	
	VAR fullName:		GSString255Hndl;
		theFile:		INTEGER;
		importWorked:	BOOLEAN;
		paramBlock:		OpenRecGS;
		myRect:			Rect;
		myPict:			PicHndl;
		myPort:			GrafPort;
		savedPort:		GrafPortPtr;
		dataLoc:		LocInfo;
		maskLoc:		LocInfo;
	
		FUNCTION ReadUnpacked(refNum: INTEGER; dstBuf: Ptr): BOOLEAN;
		{ Read a type $C1 unpacked file }
		
		VAR result:	LongInt;
		BEGIN
		  FSSetMark(refNum, 0);
		  result := FSRead(refNum, dstBuf, 32000);
		  ReadUnpacked := _toolErr = 0;
		END;
	
		FUNCTION ReadPacked02(refNum: INTEGER; dstBuf: Ptr; workBuf: Ptr): BOOLEAN;
		{ Read a type $C0 auxType $0002 packed file }
		TYPE 
		  DirEntry = RECORD
			numBytes:	INTEGER;
			modeWord:	INTEGER;
		  END;
				
		  ScanLineGroup = RECORD
			numScanLines:	INTEGER;
			scanLineDir:	ARRAY[1..200] OF DirEntry;
		  END;
				   
		  InfoBlockType = PACKED RECORD
			sizeOfBlock:		LongInt;
			idString:		String[4];
			masterMode:		INTEGER;
			pixelsPerSL:		INTEGER;
			numPalettes:		INTEGER;
			paletteArray:	ColorTable;
			{ scanline directory }
			{ packed scanlines }
		  END; {record}

		VAR 
		  curMark:			LongInt;
		  scanLine:			INTEGER;
		  bufSize:			INTEGER;
		  result:			LongInt;
		  theInfoBlock:		InfoBlockType;
		  theDir:			ScanLineGroup;
			
		BEGIN
		  curMark := 0;
		  ReadPacked02 := FALSE;
		  
		  REPEAT
			FSSetMark(refNum, curMark);	{ move to next segment	}
			IF FSRead(refNum, @theInfoBlock, SIZEOF(theInfoBlock)) <> SIZEOF(theInfoBlock)
			  THEN EXIT(ReadPacked02);
			IF theInfoBlock.idString <> 'MAIN'	{ IF not MAIN block, move to next	}
			  THEN curMark := curMark + theInfoBlock.sizeOfBlock;
		  UNTIL theInfoBlock.idString = 'MAIN';
			  
		  curMark := curMark + 15 + (theInfoBlock.numPalettes * 32); { skip over palettes	}
		  FSSetMark(refNum, curMark);
		  
		  { Read in the dir of the number of scan lines and DirEntries }
		  result := FSRead(refNum, @theDir, SizeOf(theDir));
		  IF _toolErr <> 0 THEN EXIT(ReadPacked02);
	
		  curMark := curMark + 2 + (theDir.numScanLines * SizeOf(DirEntry));
		  FSSetMark(refNum, curMark);			{ pt to 1st scan's data	}
		  
		  { Pin number of scnalines at 200 }
		  IF theDir.numScanLines > 200 THEN theDir.numScanLines := 200;
		  
		  bufSize := 32000;
		  FOR scanLine := 1 TO theDir.numScanLines DO BEGIN
			result := FSRead(refNum, workBuf, theDir.scanLineDir[scanLine].numBytes);
		    IF _toolErr <> 0 THEN EXIT(ReadPacked02);
		    result := UnpackBytes(workBuf, theDir.scanLineDir[scanLine].numBytes, dstBuf, bufSize);
		  END;
		  ReadPacked02 := TRUE;
		
		END; {ReadPacked02}
	
	
	BEGIN
	  fullName := NIL;
	  importWorked := FALSE;
	  GetPictFile := NIL;
	  
	  { We use HyperCard's 32000 byte buffer for our offscreen port }
	  GetMaskAndData(maskLoc, dataLoc);
	  
	  fullName := PToGSString(name);
	  
	  HLock(Handle(fullName));
	  paramBlock.pCount := 7;				{ Open with info so we can look	}
	  paramBlock.pathName := fullName^;		{ at file type & aux type later	}
	  paramBlock.requestAccess := readEnable;
	  paramBlock.resourceNumber := 0;
	  OpenGS(paramBlock);
	  IF _toolErr <> 0 THEN GOTO 9;
	  theFile := paramBlock.refNum;
	  	  
	  IF (paramBlock.fileType = $C1) AND (paramBlock.auxType = $0000)
		THEN importWorked := ReadUnpacked(theFile, maskLoc.ptrToPixImage)
	  ELSE IF (paramBlock.fileType = $C0) AND (paramBlock.auxType = $0002)
		THEN importWorked := ReadPacked02(theFile, maskLoc.ptrToPixImage, dataLoc.ptrToPixImage)
	  ELSE IF (paramBlock.fileType = $00) AND (paramBlock.auxType = $0000)
		THEN importWorked := ImportMacPaint(theFile, maskLoc.ptrToPixImage);
	  
	8:   
	  FSClose(theFile);
	9:
	  MyDisposeHandle(handle(fullName));
	  
	  IF importWorked THEN BEGIN
	    SetRect(myRect, 0, 0, 640, 200);
		 
		{ Create a picture from the offscreen bits }
	    savedPort := GetPort;
	    OpenPort(@myPort); 
	    myPict := OpenPicture(myRect);
	    PPtoPort(@maskLoc, myRect, 0, 0, modeCopy);
	    IF _toolErr <> 0 THEN Exit(GetPictFile);
	    ClosePicture;
	    SetPort(savedPort);
	    ClosePort(@myPort); 
	    GetPictFile := myPict;
	  END; {if}
	  
	  ChangedMaskAndData(1); 	{ Restore internal HyperCard buffers }

	END; {GetPictFile}


	FUNCTION GetClipboard: PicHndl;
	{ Get the clipboard for the contents of our window }
	VAR
	  scrapPicHndl: PicHndl;
	BEGIN
		GetClipboard := NIL;
		scrapPicHndl := PicHndl(NewHandle(1, myID, 0, NIL));
	  	GetScrap(Handle(scrapPicHndl), picScrap);
		IF _toolErr <> 0 THEN EXIT(GetClipboard);
		GetClipboard := scrapPicHndl;
	END; {GetClipboard}


	FUNCTION GetfileNResource(VAR resName: Str255): PicHndl;
	{ Create the contents of our window from a resource file }
	VAR
	  myPictureHndl:  	PicHndl;
	  homeFile:			INTEGER;
	  pictureID:		LongInt;
	  myResID:			LongInt;
	  idNameCheck: 		Str255;
	  
	BEGIN
	  GetfileNResource := NIL;
	  IF NOT FindNamedResource(rPicture, resName, homeFile, pictureID) THEN BEGIN
	  	myResID := StrToNum(resName);
		myPictureHndl := PicHndl(LoadResource(rPicture, myResID));
		IF _toolErr <> 0 THEN 
		  EXIT(GetfileNResource)
		ELSE BEGIN
		  idNameCheck := GetResourceName(rPicture, myResID);
		  IF idNameCheck = '' THEN idNameCheck := (concat ('Picture ID ', resName));
		  resName := idNameCheck;	
		  pictureID := myResID;
		END {else}
	  END ELSE BEGIN
		myPictureHndl := PicHndl(LoadResource(rPicture, pictureID));
		IF _toolErr <> 0 THEN EXIT(GetfileNResource)
	  END;  {else}
	  DetachResource(rPicture, pictureID);
	  IF _toolErr <> 0 THEN BEGIN
	    ReleaseResource(3, rPicture, pictureID);
	    EXIT(GetfileNResource);
      END; {if}
	  { Return the handle to the picture }
	  GetfileNResource := myPictureHndl;
	END; {GetfileNResource}
	
	
	FUNCTION CorrectVersion: BOOLEAN;
	{ This returns true IF the version of HyperCard is >= minVersion }
	CONST
	  minVersion = '1.1';
	VAR
	  tempHandle:	Handle;
	  tempStr:		Str255;
	BEGIN
	  tempHandle := EvalExpr('the version');
	  ZeroToPas(tempHandle^, tempStr);
	  MyDisposeHandle(tempHandle);
	  CorrectVersion := tempStr >= minVersion;
	END; {CorrectVersion}
	
	
	FUNCTION SetWindowRect(whatStyle: INTEGER): Rect;
	VAR
	  h1Border:	INTEGER;
	  v1Border:	INTEGER;
	  h2Border:	INTEGER;
	  v2Border:	INTEGER;
	  wndHOrg:	INTEGER;
	  wndVOrg:	INTEGER;
	  tempRect: Rect;
	BEGIN
	  { Figure adjustments for various window styles }
	  h1Border := 4; h2Border := 636;
	  v1Border := 2; v2Border := 198;
	  CASE whatStyle OF
	    xWindoidStyle: BEGIN
		  v1Border := 8;
		END; {xWindoidStyle}
		  
	    xDialogStyle: BEGIN
		  h1Border := 14; h2Border := 626;
		  v1Border := 7; v2Border := 193;
		END; {xDialogStyle}
		  
	    xShadowStyle: BEGIN
		  h2Border := 634; v2Border := 197;
		END; {xShadowStyle}
	  END; {case}
	  
	  { Default location is centered on screen }
	  wndHOrg := BAND(320 - picWidth DIV 2, $FFFE);
	  wndVOrg := 100 - picHeight DIV 2;
	  SetRect(tempRect, wndHOrg, wndVOrg, picWidth + wndHOrg, picHeight + wndVOrg);

	  { Keep the window frame on the screen }
	  WITH tempRect DO BEGIN
	    IF left < h1Border THEN left := h1Border;
	    IF right > h2Border THEN right := h2Border;
	    IF top < v1Border THEN top := v1Border;
	    IF bottom > v2Border THEN bottom := v2Border;
	  END; {with}
	  
	  { If the menu bar is visible, keep top position below it }
	  IF (GetSysBar^^.ctlFlag <= 127) AND (tempRect.top < v1Border + 13) THEN 
	    tempRect.top := v1Border + 13;

	  SetWindowRect := tempRect;
	END; {SetWindowRect}

BEGIN
	pCount := paramPtr^.paramCount;
	myID := paramPtr^.userID;
	visible := TRUE;
	hasScrollBars := FALSE;
	
	{ If the paramCount is negative, we have been called in response to an event }
	IF pCount < 0 THEN BEGIN
	  HandleEvents;
	  EXIT(XPicture);
	END; {IF}

	IF (pCount < 0) OR (pCount > 5) THEN HTError;
	
	{ Make sure we are running at least version 1.1 of HyperCard IIGS }
    IF NOT CorrectVersion THEN ReturnResult(WrongVersionStr);
	
	IF pCount = 0 THEN BEGIN
	  SendHCmessage('Answer file "Display what picture?" of type Paint');
	  nameHandle := evalExpr('it');
	  ZeroToPas(nameHandle^, str);
	  MyDisposeHandle(nameHandle);
	  IF str = '' THEN BEGIN
		ReturnResult('Cancel');
		EXIT(XPicture) ;
	  END;
	  SendHCMessage(WaitCursorStr);
	  myPicHandle := GetPictFile(str);
	END ELSE BEGIN
	  ZeroToPas(paramPtr^.params[1]^, str);
	
	  { Display help or copyright info in response to "?" or "!" }
	  IF paramPtr^.paramCount = 1 THEN BEGIN
	    IF str = '!' THEN CommandInfo(CopyrightStr);
	    IF str = '?' THEN CommandInfo(HelpStr);
	  END; {IF}
		
	  { Create the contents for the window }
	  SendHCMessage(WaitCursorStr);
	  IF pCount > 1 THEN BEGIN
		ZeroToPas(paramPtr^.params[2]^, tempStr);	
		IF StringEqual(tempStr, FileStr) THEN BEGIN
		  myPicHandle := GetPictFile(str);  { get pict from file }
		  IF myPicHandle = NIL THEN ReturnResult(NoFileStr);
		END ELSE IF StringEqual(tempStr, ResourceStr) THEN BEGIN
		  myPicHandle := GetfileNResource(Str);  { get pict from resource }
		  IF myPicHandle = NIL THEN ReturnResult(NoResourceStr);
		END ELSE IF StringEqual(tempStr, ClipStr) THEN BEGIN
		  myPicHandle := GetClipboard;  { get pict from Clipboard }
		  IF myPicHandle = NIL THEN ReturnResult(NoClipStr);
		END ELSE HTError
	  END ELSE BEGIN
		myPicHandle := GetPictFile(str);  { Default to file }
		IF myPicHandle = NIL THEN ReturnResult(NoFileStr);
	  END; {else}
	END; {else}
	
	{ Determine initial visibility of the XWindow }
	IF pCount > 3 THEN BEGIN
	  ZeroToPas(paramPtr^.params[4]^, tempStr);
	  visible := StrToBool(tempStr);
	END;
	
	{ Translate the style string, if it exists, into a window style }
	IF pCount < 3 THEN BEGIN
	  hasScrollBars := TRUE;
	  windowStyle := xWindoidStyle;
	END ELSE BEGIN
	  ZeroToPas(paramPtr^.params[3]^, tempStr);
	  IF StringEqual(tempStr, RectStr1) OR StringEqual(tempStr, RectStr2) THEN
	    windowStyle := xRectStyle
	  ELSE IF StringEqual(tempStr, ShadowStr) THEN
	    windowStyle := xShadowStyle
	  ELSE IF StringEqual(tempStr, DialogStr) THEN
	    windowStyle := xDialogStyle
	  ELSE IF StringEqual(tempStr, WindoidStr) THEN
	    windowStyle := xWindoidStyle
	  ELSE BEGIN
	    windowStyle := xWindoidStyle;
		hasScrollBars := TRUE;
	  END; {else}
	END; {else}
	
	picWidth := GetWindowWth(myPicHandle, hasScrollBars);
	picHeight := GetWindowHt(myPicHandle, hasScrollBars);
		
	{ Get the rect param if it exists, and set the window rect, otherwise get the
	  rect from the pict and center it on the screen }
	  
	IF pCount > 4 THEN BEGIN
	  ZeroToPas(paramPtr^.params[5]^, tempStr);
	  IF tempStr = '' THEN windRect := SetWindowRect(windowStyle)
	  ELSE BEGIN
	    StrToRect(tempStr, windRect);
		windRect.left := windRect.left * 2;		{ Adjust to 640 coords }
		windRect.right := windRect.right * 2;
	  END
	END ELSE 
	  windRect := SetWindowRect(windowStyle);
	
	{ Create window }
	picXWindow := NewXWindow(windRect, str, visible, windowStyle);   		
	IF picXWindow = NIL THEN ReturnResult(CreateErrStr);
	
	{ Create a record to store the information about our window }
	wData := wDataHndl(NewHandle(SIZEOF(WdataRec), myID, 0, NIL));
	IF _toolErr <> 0 THEN HandleErrors('Memory allocation Error');
		
	HLock(Handle(wData));
	WITH wData^^ DO BEGIN
	  IF hasScrollBars THEN BEGIN
	    vScroll := NewVScroll(picXWindow, myPicHandle, 0);
	    hScroll := NewHScroll(picXWindow, myPicHandle, 0);
	    growbox := NewGrowbox(picXWindow);
	  END ELSE BEGIN
	    vScroll := NIL;
	    hScroll := NIL;
	    growbox := NIL;
	  END; {else}
	  myPictureHndl := myPicHandle;
	  windowType := windowStyle;
	  usesScrollBars := hasScrollBars;
	END; {WITH}
	HUnlock(Handle(wData));
	
	{ Save the handle to our window record }
	SetXWindowValue(picXWindow, LongInt(wData));
	
  END;

END.
