/*
	trap.c
	Functions to support testing for traps
	and testing for Gestalt features.
	
	11-13-90 1.0d3 JRP	original
*/

#include	<Types.h>
#include	<Traps.h>
#include	<OSUtils.h>
#include	<GestaltEqu.h>

#include	"trap.pro"

short aliasAvailable(void)
/*
	Returns true if Alias is available.
*/
{
	if(gestaltAvailable())
		return(Gestalt(gestaltAliasMgrAttr, gestaltAliasMgrPresent)==noErr);
	else
		return false;
}	/*	--------------------------------------------	aliasAvailable		*/

short gestaltAvailable(void)
/*
	Returns true if gestalt is available.
	From IM-VI. chap 3, page 8
*/
{
	#define	_Gestalt	0xA1AD
	
	return(trapAvailable(_Gestalt));
}	/*	--------------------------------------------	gestaltAvailable		*/

short getTrapType(theTrap)
/*
	Returns the trap type.
	From IM-VI. chap 3, page 8
*/
	short	theTrap;
{
	#define	trapMask	0x0800
	
	if(theTrap & trapMask)
		return ToolTrap;
	else
		return OSTrap;
}	/*	--------------------------------------------	getTrapType		*/

short numToolboxTraps(void)
/*
	Returns the number of toolbox traps.
	From IM-VI. chap 3, page 8
*/
{
	if(NGetTrapAddress(_InitGraf, ToolTrap)==
			NGetTrapAddress(0xAA6E, ToolTrap))
				return 0x200;
	else
		return 0x400;
}	/*	--------------------------------------------	numToolboxTraps		*/

short trapAvailable(theTrap)
/*
	Returns true if the trap is available.
	From IM-VI. chap 3, page 8
*/
	short	theTrap;
{
	short	tType;

	tType = getTrapType(theTrap);
	if(tType==ToolTrap)
	{
		theTrap = theTrap & 0x07FF;
		if(theTrap>=numToolboxTraps())
			theTrap = _Unimplemented;
	}
	return(NGetTrapAddress(theTrap, tType)!=
				NGetTrapAddress(_Unimplemented, ToolTrap));
}	/*	--------------------------------------------	trapAvailable	*/

