/*
	ColorScreen XCMD.  
	
	1990 Apple Computer, Inc.; by Mike Byrne
	
	Simply put, it changes the forecolor and (if desired) the backcolor that
	HyperCard uses to draw in the current Port.  It's based on "old" QuickDraw, that is, it doesn't force 
	HyperCard to use a cGrafPort, so memory useage is minimal (then again, so is the number of colors).
	Sorta useless on a B+W machine...
	
	Form:
	ColorScreen <forecolor>, [<backcolor>]
	
	Colors supported by ColorScreen:
	- black                 - blue
	- white                 - cyan
	- red                   - magenta
	- green                 - yellow

	ColorScreen will accept all lower case, all upper case, or capitalized strings for the color spec.  
	That is, "black"  "Black"  and "BLACK" all work, but not "bLack."
		
	# the MPW 3.2 build commands:
	C -b ColorScreen.c -mbg off
		Link -w -rt XCMD=606 
			-m ENTRYPOINT 
			-sg ColorScreen 
			ColorScreen.c.o 
			"{Libraries}HyperXLib.o" 
			"{Libraries}Runtime.o" 
			"{Libraries}Interface.o" 
			"{CLibraries}StdCLib.o" 
			-o "some stack"
*/

#include <Types.h>
#include <Windows.h>
#include <string.h>
#include <CType.h>
#include <Memory.h>
#include <QuickDraw.h>
#include "HyperXCmd.h"

#define NIL (long) 0
#define NULL (long) 0

/* prototypes */
void 	ErrorBack(XCmdPtr paramPtr, char *message);
void 	MoveLockParams ( XCmdPtr paramPtr, short paramCount );
void 	UnlockParams  ( XCmdPtr paramPtr, short paramCount );
short 	SwitchColor( char* ask );

pascal void EntryPoint(XCmdPtr paramPtr)
{

	short		color;
	WindowPtr	wp;
	
	/* move high and lock the parameters. */
	MoveLockParams(paramPtr, paramPtr->paramCount);

	/* check for copyright or syntax help request */
	if (!strcmp( (char*)*paramPtr->params[0], "!") ) {
		ErrorBack(paramPtr, "v1.1, 1990 Apple Computer, Inc.; by Mike Byrne");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	} else if (!strcmp ( (char*)*paramPtr->params[0], "?") ) {
		ErrorBack(paramPtr, "ColorScreen syntax is 'ColorScreen <forecolor>, [<backcolor>]'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* not a copyright or help request.       */     
	/* check for correct number of parameters */
	if (paramPtr->paramCount > 2) {
		ErrorBack(paramPtr, "ColorScreen syntax is 'ColorScreen <forecolor>, [<backcolor>]'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	if (color = SwitchColor( (char*)*paramPtr->params[0])) {
		/* the color set is cool.  Set the forecolor. */
		GetPort(&wp);
		SetPort(wp);
		ForeColor(color);
		if (color = SwitchColor( (char*)*paramPtr->params[1])) {
			BackColor(color);
		}
		InvalRect( &(wp->portRect) ); 
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	} else {
		ErrorBack(paramPtr, "ColorScreen doesn't recognize that color");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}
}



short SwitchColor( char* ask )
{
	short 	i;
	
	/* convert the string to uppercase */
	for (i=0; i < strlen(ask); i++) {
		ask[i] = toupper(ask[i]);
	}
	
	/* switch on the string. */
	if (!strcmp(ask, "BLACK") || !strcmp(ask, "1")) 
		return(blackColor);
	else if (!strcmp(ask, "WHITE") || !strcmp(ask, "2")) 
		return(whiteColor);
	else if (!strcmp(ask, "RED")  || !strcmp(ask, "3")) 
		return(redColor);
	else if (!strcmp(ask, "GREEN") || !strcmp(ask, "4")) 
		return(greenColor);
	else if (!strcmp(ask, "BLUE") || !strcmp(ask, "5")) 
		return(blueColor);
	else if (!strcmp(ask, "CYAN") || !strcmp(ask, "6")) 
		return(cyanColor);
	else if (!strcmp(ask, "MAGENTA") || !strcmp(ask, "7")) 
		return(magentaColor);
	else if (!strcmp(ask, "YELLOW") || !strcmp(ask, "8")) 
		return(yellowColor);
	else return(0);
}
	
	
void ErrorBack(XCmdPtr paramPtr, char *message)
{
	Handle  mesHnd;

	/*
		Allocate space for an error message.
		Copy the string into it.
		Return the handle to HyperCard.
	*/
	mesHnd = NewHandle((long)(strlen(message)+1));
	if (mesHnd == nil) return;
	strcpy((char *)*mesHnd,message);
	paramPtr->returnValue = mesHnd;
}


/*  move high and lock down all parameters  
    ----------------------------------------------------------------------- */
void MoveLockParams ( XCmdPtr paramPtr, short paramCount )
{
	short i;
	
	for(i=0; i <= paramCount-1; i++)
	{
		MoveHHi(paramPtr->params[i]);
		HLock(paramPtr->params[i]);
	}
}



/* unlock all parameter handles in the XCmdBlock  
   ---------------------------------------------  */
void UnlockParams  ( XCmdPtr paramPtr, short paramCount )
{	short i;
	
	for(i=0; i <= paramCount-1; i++)
		{ HUnlock(paramPtr->params[i]);}
}
