/*
	ColonToComma XFCN v1.2
	
	1991 Apple Computer, Inc.; by Mike Byrne
	
	Takes a string of the form "mm:ss" or "mm:ss:bb" and converts it to "mm,ss,00" or "mm,ss,bb".
	
	Form:
	ColonToComma(<string>)
	
	# the MPW 3.2 build commands:
	C -b ColonToComma.c -mbg off
		Link -w -t STAK -c WILD -rt XFCN=700 
			-m ENTRYPOINT 
			-sg ColonToComma 
			ColonToComma.c.o 
			"{Libraries}HyperXLib.o" 
			"{Libraries}Runtime.o" 
			"{Libraries}Interface.o" 
			"{CLibraries}StdCLib.o" 
			-o "teststack"
*/

#include <Types.h>
#include <string.h>
#include <Memory.h>
#include "HyperXCmd.h"

#define NULL (long) 0
#define NIL (long) 0

#define kNumParams 1


/* prototypes */
void ErrorBack(XCmdPtr paramPtr, char *message);
void MoveLockParams ( XCmdPtr paramPtr, short paramCount );
void UnlockParams  ( XCmdPtr paramPtr, short paramCount );



pascal void EntryPoint(XCmdPtr paramPtr)
{

	/* variable declarations */
	char	newString[50];
	short	length;
	short	replacements = 0;
	short	i = 0;

	/* move high and lock the parameters. */
	MoveLockParams(paramPtr, paramPtr->paramCount);

	/* check for copyright or syntax help request */
	if (!strcmp( (char*)*paramPtr->params[0], "!") ) {
		ErrorBack(paramPtr, "v1.2, 1991 Apple Computer, Inc.; by Mike Byrne");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	} else if (!strcmp ( (char*)*paramPtr->params[0], "?") ) {
		ErrorBack(paramPtr, "ColonToComma syntax is 'ColonToComma(<string>)'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* not a copyright or help request.       */     
	/* check for correct number of parameters */
	if (paramPtr->paramCount != kNumParams) {
		ErrorBack(paramPtr, "Error: ColonToComma syntax is 'ColonToComma(<string>)'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* copy the parameter to orgString */
	strcpy(newString, (char*)*paramPtr->params[0]);
	length = strlen(newString);
	
	/* loop through, replacing colons with commas */
	for ( i = 0 ; i < 10 && i < length; i++) {
		if (newString[i] == ':') {
			newString[i] = ',';
			replacements++;
		} 
	}
	
	/* now, if only one replacement was made, we should add zeros. */
	if (replacements == 1) {
		strcat(newString, ",00");
	}
	
	
	/* we're outta here */
	ErrorBack(paramPtr, newString);
	UnlockParams(paramPtr, paramPtr->paramCount);
	return;


}





	
/* allocate and load up paramPtr->returnValue with a string 
   -------------------------------------------------------- */
void ErrorBack(XCmdPtr paramPtr, char *message)
{
	Handle  mesHnd;

	/*
		Allocate space for an error message.
		Copy the string into it.
		Return the handle to HyperCard.
	*/
	mesHnd = NewHandle((long)(strlen(message)+1));
	if (mesHnd == nil) return;
	strcpy((char *)*mesHnd,message);
	paramPtr->returnValue = mesHnd;
}



/*  move high and lock down all parameters  
    ----------------------------------------------------------------------- */
void MoveLockParams ( XCmdPtr paramPtr, short paramCount )
{
	short i;
	
	for(i=0; i <= paramCount-1; i++)
	{
		MoveHHi(paramPtr->params[i]);
		HLock(paramPtr->params[i]);
	}
}




/* unlock all parameter handles in the XCmdBlock  
   ---------------------------------------------  */
void UnlockParams  ( XCmdPtr paramPtr, short paramCount )
{	short i;
	
	for(i=0; i <= paramCount-1; i++)
		{ HUnlock(paramPtr->params[i]);}
}
