/*
	FilterDirectory XFCN v1.3
	
	by Mike Byrne,  1990-1 Apple Computer, Inc.
	
	This XFCN will take a pathname and traverse the entire directory, looking for files of the
	types specified by the sets of filters provided by the last parameters (up to fourteen types).
	If the second parameter is anything but "false" (or any of the following: "F", "f", "False",
	"FALSE", "0", you get the idea), the XFCN will prefix the full pathname of the file with the 
	creator and type of the file, separating the two with a comma.
	
	Form:
	FilterDirectory(<pathname>, <t/f for creator and type>, <filter1>, [<filter2>, <f3>....])
	
	Example:
	put FilterDirectory("Hard Disk:", true, "TEXT", "WDBN") into textFiles
	-- this returns a list like:
	MPS TEXT,Hard Disk:MPW:source code.c
	MACATEXT,Hard Disk:Read Me
	MSWDWDBN,Hard Disk:Word Files:file1
	MSWDWDBN,Hard Disk:Word Files:file2
	
	Note:
	If scanning large directories that contain many files, the return string can get quite large.
	Thus, the result of this XFCN should always be put into a variable and a not a field, as it may
	not fit into a field.
	
	
	# the MPW 3.2 build commands:
	C -b FilterDirectory.c -mbg off
		Link -w -t STAK -c WILD -rt XFCN=605 
			-m ENTRYPOINT 
			-sg FilterDirectory 
			FilterDirectory.c.o 
			"{Libraries}HyperXLib.o" 
			"{Libraries}Runtime.o" 
			"{Libraries}Interface.o" 
			"{CLibraries}StdCLib.o" 
			-o "teststack"
*/

#include <Types.h>
#include <OSUtils.h>
#include <Files.h>
#include <string.h>
#include <Memory.h>
#include <Packages.h>
#include <Errors.h>
#include <Events.h>
#include "HyperXCmd.h"
#define NULL  0L
#define NIL  0L

#define kMaxSize 20000


/* prototypes */  
void ErrorBack(XCmdPtr paramPtr, char *message);
void MoveLockParams ( XCmdPtr paramPtr, short paramCount );
void UnlockParams  ( XCmdPtr paramPtr, short paramCount );
void ExpandSubdir(XCmdPtr paramPtr, HFileInfo* infoRec, long searchMe, char* builtPath, Boolean tpCrFlag,
					long* fileCount);
void AddFile(XCmdPtr paramPtr, HFileInfo* infoRec, char* builtPath, Boolean	tpCrFlag, long* fileCount);
void GrowRetStr(XCmdPtr paramPtr);
Boolean CancelRequest(XCmdPtr paramPtr);


pascal void EntryPoint(XCmdPtr paramPtr)
{
	char		orgVol[34];
	short		numParams, i;
	char		dirPath[301];
	HFileInfo	infoRec;
	short		orgVRefNum=0;
	long		searchMe;
	Boolean		tpCrFlag;
	long		fileCount;
	
	numParams = paramPtr->paramCount;
	
	/*  if the params are read only, move high and lock them... */
	MoveLockParams(paramPtr, numParams);

	/* check for syntax or copyright help request */
	if (!strcmp( (char*)*paramPtr->params[0], "!") ) {
		ErrorBack(paramPtr, "v1.3, 1990-1 Apple Computer, Inc.; by Mike Byrne");
		UnlockParams(paramPtr, numParams);
		return;
	} else if (!strcmp ( (char*)*paramPtr->params[0], "?") ) {
		ErrorBack(paramPtr, 
		"FilterDirectory syntax is 'FilterDirectory(<pathname>, <t/f crtrType>, <filter1>, [<filter2>, <f3>....])'");
		UnlockParams(paramPtr, numParams);
		return;
	}
	
	/* this is not a test.  this is an actual request for the files. */
	/* check the second param to see if the user wants the type/creator info. */
	/* first, convert the flag parameter to uppercase. */
	for (i=0; i < strlen((char*)*paramPtr->params[1]); i++) {
		((char*)*paramPtr->params[1])[i] = toupper(((char*)*paramPtr->params[1])[i]);
	}
	
	/* set the flag */
	if ( (!strcmp((char*)*paramPtr->params[1], "FALSE")) || (!strcmp((char*)*paramPtr->params[1], "0"))
	|| (!strcmp((char*)*paramPtr->params[1], "F")) ) {
		tpCrFlag = false;
	} else {
		tpCrFlag = true;
	}
	
	
	
	/* pull out the volume name */
	for (i=0; ((*(paramPtr->params[0]))[i] != ':') && (i < 33); i++) 
		{ orgVol[i] = (*(paramPtr->params[0]))[i]; }
	orgVol[i] = ':';
	orgVol[i+1] = '\0'; 
	c2pstr(orgVol);

	/* set the default volume */
	if (SetVol(orgVol, 0) != noErr) {											// toolbox
		ErrorBack(paramPtr, "Error:  Could not set the default origin volume");
		UnlockParams(paramPtr, numParams);
		return;
	}
	
	/* get the full pathname and make it a pascal string. */
	strcpy(dirPath, (char*)*paramPtr->params[0]);
	c2pstr(dirPath);

	/* now, set up the HFblock.*/
	infoRec.ioNamePtr = &dirPath;
	infoRec.ioFDirIndex = 0;

	/* get the directory ID. */
	if ( PBGetCatInfo((CInfoPBPtr)&infoRec, false) != noErr) {					// toolbox
		ErrorBack(paramPtr, "Error:  Cannot get information about the requested directory.");
		UnlockParams(paramPtr, numParams);
		return;
	}

	/* allocate space for the return value. */
	if ((paramPtr->returnValue = NewHandleClear(kMaxSize)) == NIL) {			// toolbox
		ErrorBack(paramPtr,"Error:  Not enough memory to complete");
		UnlockParams(paramPtr, numParams);
		return;
	}
	
	/* set the Volume Reference Number to 0 (the default volume, set earlier) and traverse. */
	searchMe = infoRec.ioDirID;
	infoRec.ioVRefNum = fileCount = 0;
	ExpandSubdir(paramPtr, &infoRec, searchMe, (char*)*paramPtr->params[0], tpCrFlag, &fileCount);
	
	/* okay, that's it.  we're outta here. */
	SendHCMessage(paramPtr,"\pset cursor to normal");
	if (CancelRequest(paramPtr)) { ErrorBack(paramPtr, "Scan cancelled."); };
	UnlockParams(paramPtr, numParams);
	return;
	
}


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	ExpandSubdir is the recursive bit that hacks apart subdirectories.   */

void ExpandSubdir(XCmdPtr paramPtr, HFileInfo* infoRec, long searchMe, char* builtPath, Boolean tpCrFlag,
					long* fileCount)
{
	char		passDown[350];
	short		fileIndex = 0;
	OSErr		theError = noErr;

	strcpy(passDown, builtPath);
	
	if ( !strcmp((char*)*paramPtr->returnValue, "Scan cancelled.") )   { return; }
	
	do
	{
		fileIndex++;

		infoRec->ioFDirIndex = fileIndex;       
		infoRec->ioDirID = searchMe;     
		theError = PBGetCatInfo((CInfoPBPtr)infoRec,false);						/* toolbox */
		 if (theError == noErr) 
		 {
			 /* check the file attributes for folderhood */
			 if (((infoRec->ioFlAttrib>>4) & 0x01) == 1)
	
			 { /*we have a dir*/	
			 	p2cstr(infoRec->ioNamePtr);
				strcat(passDown, infoRec->ioNamePtr);
				c2pstr(infoRec->ioNamePtr);
				strcat(passDown, ":");
				ExpandSubdir(paramPtr, infoRec, infoRec->ioDirID, passDown, tpCrFlag, fileCount);     /* recurse */
				strcpy(passDown, builtPath);
				theError = 0;     
			 } else { /*we have a file*/
				AddFile(paramPtr, infoRec, passDown, tpCrFlag, fileCount);
				strcpy(passDown, builtPath);
			 	if ( ((*fileCount)%13) == 1) {
					SendHCMessage(paramPtr, "\pset cursor to busy"); 
				}   
			 } /*else*/
		} 
	} while ( (theError == noErr) && !CancelRequest(paramPtr) );
} 


/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	AddFile checks the file type, and, if it finds a match, adds the full path to the returnValue.  */
	
void AddFile(XCmdPtr paramPtr, HFileInfo* infoRec, char* builtPath, Boolean tpCrFlag, long* fileCount)
{
	short 		i;
	long		checkType;
	OSType		fileType = infoRec->ioFlFndrInfo.fdType;
	OSType		fileCrtr = infoRec->ioFlFndrInfo.fdCreator;
	Boolean		check = false;
	char		type[5];
	char		creator[5];
	
	
	/* look to see if the user cancelled the search.  If so, we're outta here... */
 	if ( CancelRequest(paramPtr) ) { return; }
	
	/*  check to see that the file is of one of the types specified by the filters.   Since the
		finder (fileType) info is in an OSType (a long) and the parameter is a string, use 
		BlockMove to put the string into a long for comparison.								*/
	for(i = 2; i <= paramPtr->paramCount; i++) {
		BlockMove( *(paramPtr->params[i]), (Ptr)&checkType, sizeof(OSType));		// toolbox
		if ( (fileType != NULL) && (checkType == (long)fileType) ) 
			{ check = true; }
	}
	
	/* if it is of one of the types, add it in there. */
	if (check) {
		/* check to see if we need to grow the return param. */
		GrowRetStr(paramPtr);
		
		/* if creator and type info was requested, give it to 'em.  (Again, need to use BlockMove
			to move the OSType stuff from the finder info into strings.  The stings won't 
			 be zero-terminated after the block move, so tack zeros on the end.  				*/
			 
		if (tpCrFlag) {
			ZeroBytes(paramPtr, type, 5);											// glue
			ZeroBytes(paramPtr, creator, 5);										// glue
			BlockMove((Ptr)&fileCrtr, creator, sizeof(OSType));						// toolbox
			creator[4] = '\0';
			strcat((char*)*paramPtr->returnValue, creator);
			BlockMove((Ptr)&fileType, type, sizeof(OSType));						// toolbox
			strcat((char*)*paramPtr->returnValue, type); 
			type[4] = '\0'; 
			strcat((char*)*paramPtr->returnValue, ",");
		}

		/* catenate the path- and filename to the return string. */
		strcat( (char*)*paramPtr->returnValue, builtPath);
		p2cstr(infoRec->ioNamePtr);
		strcat( (char*)*paramPtr->returnValue, infoRec->ioNamePtr);
		c2pstr(infoRec->ioNamePtr);
		
		/* and, finally, add a return to the end of the line and increment the counter. */
		strcat( (char*)*paramPtr->returnValue, "\n");
		(*fileCount)++;
	}
}


/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	GrowRetStr checks the size of the string in the returnValue.  If
	it appears that it is getting near the handle size, the handle is
	grown to accomodate lots more...							   */
	
void GrowRetStr(XCmdPtr paramPtr)
{
	long	size = strlen((char*)*paramPtr->returnValue);
	if (size > (kMaxSize-1000) ) {
		size += (kMaxSize-1000);
		SetHandleSize(paramPtr->returnValue, (Size)size);					// toolbox
	}
}



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	CancelRequest checks to see if the last thing in the event queue is a command-period.  Obviously,
	this is used to check for a user cancelling the scan. */
	
Boolean CancelRequest(XCmdPtr paramPtr) 
{
	char		theKey;
	EventRecord lastEvent;

	if (EventAvail(keyDownMask, &lastEvent)) {
		if ((lastEvent.modifiers & cmdKey))	{			/* Command key down.	*/
			theKey = (char)(lastEvent.message & charCodeMask);
			if ( theKey == '.' ) {				/* Period hit.			*/
				ErrorBack(paramPtr, "Scan cancelled."); 
				return true;
				}
			}
		}

	return false;
	
}



	
/* allocate and load up paramPtr->returnValue with a string 
   -------------------------------------------------------- */
void ErrorBack(XCmdPtr paramPtr, char *message)
{
	Handle  mesHnd;

	/*
		Allocate space for an error message.
		Copy the string into it.
		Return the handle to HyperCard.
	*/
	mesHnd = NewHandle((long)(strlen(message)+1));
	if (mesHnd == nil) return;
	strcpy((char *)*mesHnd,message);
	paramPtr->returnValue = mesHnd;
}



/*  move high and lock down all parameters  
    ----------------------------------------------------------------------- */
void MoveLockParams ( XCmdPtr paramPtr, short paramCount )
{
	short i;
	
	for(i=0; i <= paramCount-1; i++)
	{
		MoveHHi(paramPtr->params[i]);
		HLock(paramPtr->params[i]);
	}
}




/* unlock all parameter handles in the XFCNBlock  
   ---------------------------------------------  */
void UnlockParams  ( XCmdPtr paramPtr, short paramCount )
{	short i;
	
	for(i=0; i <= paramCount-1; i++)
		{ HUnlock(paramPtr->params[i]);}
}
